@@ -14,6 +14,8 @@
 
 #include <Libkleo/GnuPG>
 
+#include <QLocale>
+
 #include <KLocalizedString>
 #include <KLazyLocalizedString>
 static const char kleopatra_version[] = KLEOPATRA_VERSION_STRING;
@@ -54,7 +56,9 @@ static const about_data credits[] = {
 };
 
 AboutData::AboutData()
-    : KAboutData(QStringLiteral("kleopatra"), i18n("Kleopatra"),
+    : KAboutData(QStringLiteral("kleopatra"),
+      (Kleo::brandingWindowTitle().isEmpty() ?
+       i18n("Kleopatra") : Kleo::brandingWindowTitle()),
 #ifdef Q_OS_WIN
                  Kleo::gpg4winVersion(),
                  Kleo::gpg4winDescription(),
@@ -91,7 +95,17 @@ AboutData::AboutData()
         setVersion(Kleo::gpg4winVersion().toUtf8());
         setShortDescription(Kleo::gpg4winDescription());
         setOtherText(Kleo::gpg4winLongDescription());
-        setBugAddress("https://dev.gnupg.org/u/rgpg4win");
+
+        /* Bug reporting page is only available in german and english */
+        if (QLocale().uiLanguages().first().startsWith(QStringLiteral("de"))) {
+            setBugAddress("https://gnupg.com/vsd/report.de.html");
+        } else {
+            setBugAddress("https://gnupg.com/vsd/report.html");
+        }
+    } else {
+#ifdef Q_OS_WIN
+       setBugAddress("https://dev.gnupg.org/u/rgpg4win");
+#endif
     }
 
     const auto backendVersions = Kleo::backendVersionInfo();
@@ -103,7 +117,4 @@ AboutData::AboutData()
                      QLatin1String{"</li></ul>"} +
                      otherText());
     }
-#ifdef Q_OS_WIN
-    setBugAddress("https://dev.gnupg.org/u/rgpg4win");
-#endif
 }
