@@ -11,14 +11,16 @@
 #include <config-kleopatra.h>
 
 #include "certifycertificatecommand.h"
-#include "newcertificatecommand.h"
+#include "newopenpgpcertificatecommand.h"
 
 #include "command_p.h"
 
 #include "exportopenpgpcertstoservercommand.h"
 #include "dialogs/certifycertificatedialog.h"
+#include "utils/keys.h"
 #include "utils/tags.h"
 
+#include <Libkleo/Algorithm>
 #include <Libkleo/KeyCache>
 #include <Libkleo/Formatting>
 
@@ -30,8 +32,9 @@
 
 #include <gpgme++/key.h>
 
-#include <KLocalizedString>
 #include "kleopatra_debug.h"
+#include <KLocalizedString>
+#include <kwidgetsaddons_version.h>
 
 using namespace Kleo;
 using namespace Kleo::Commands;
@@ -61,6 +64,7 @@ private:
     void createJob();
 
 private:
+    GpgME::Key target;
     std::vector<UserID> uids;
     QPointer<CertifyCertificateDialog> dialog;
     QPointer<QGpgME::SignKeyJob> job;
@@ -138,13 +142,24 @@ CertifyCertificateCommand::~CertifyCertificateCommand()
 
 void CertifyCertificateCommand::doStart()
 {
-
     const std::vector<Key> keys = d->keys();
     if (keys.size() != 1 ||
             keys.front().protocol() != GpgME::OpenPGP) {
         d->finished();
         return;
     }
+    // hold on to the key to certify to avoid invalidation during refreshes of the key cache
+    d->target = keys.front();
+
+    if (d->target.isExpired() || d->target.isRevoked()) {
+        const auto title = d->target.isRevoked() ? i18nc("@title:window", "Key is Revoked") : i18nc("@title:window", "Key is Expired");
+        const auto message = d->target.isRevoked() //
+            ? i18nc("@info", "This key has been revoked. You cannot certify it.")
+            : i18nc("@info", "This key has expired. You cannot certify it.");
+        d->information(message, title);
+        d->finished();
+        return;
+    }
 
     auto findAnyGoodKey = []() {
         const std::vector<Key> secKeys = KeyCache::instance()->secretKeys();
@@ -155,18 +170,27 @@ void CertifyCertificateCommand::doStart()
     };
 
     if (!findAnyGoodKey()) {
+#if KWIDGETSADDONS_VERSION >= QT_VERSION_CHECK(5, 100, 0)
+        auto sel =
+            KMessageBox::questionTwoActions(d->parentWidgetOrView(),
+#else
         auto sel = KMessageBox::questionYesNo(d->parentWidgetOrView(),
-                    xi18nc("@info", "To certify other certificates, you first need to create an OpenPGP certificate for yourself.") +
-                    QStringLiteral("<br><br>") +
-                    i18n("Do you wish to create one now?"),
-                    i18n("Certification Not Possible"));
+#endif
+                                            xi18nc("@info", "To certify other certificates, you first need to create an OpenPGP certificate for yourself.")
+                                                + QStringLiteral("<br><br>") + i18n("Do you wish to create one now?"),
+                                            i18n("Certification Not Possible"),
+                                            KGuiItem(i18n("Create")),
+                                            KStandardGuiItem::cancel());
+#if KWIDGETSADDONS_VERSION >= QT_VERSION_CHECK(5, 100, 0)
+        if (sel == KMessageBox::ButtonCode::PrimaryAction) {
+#else
         if (sel == KMessageBox::Yes) {
+#endif
             QEventLoop loop;
-            auto cmd = new Commands::NewCertificateCommand();
+            auto cmd = new NewOpenPGPCertificateCommand;
             cmd->setParentWidget(d->parentWidgetOrView());
-            cmd->setProtocol(GpgME::OpenPGP);
             connect(cmd, &Command::finished, &loop, &QEventLoop::quit);
-            QMetaObject::invokeMethod(cmd, &Commands::NewCertificateCommand::start, Qt::QueuedConnection);
+            QMetaObject::invokeMethod(cmd, &NewOpenPGPCertificateCommand::start, Qt::QueuedConnection);
             loop.exec();
         } else {
             Q_EMIT(canceled());
@@ -196,11 +220,10 @@ void CertifyCertificateCommand::doStart()
     d->ensureDialogCreated();
     Q_ASSERT(d->dialog);
 
-    Key target = d->key();
-    if (!(target.keyListMode() & GpgME::SignatureNotations)) {
-        target.update();
+    if (!(d->target.keyListMode() & GpgME::SignatureNotations)) {
+        d->target.update();
     }
-    d->dialog->setCertificateToCertify(target);
+    d->dialog->setCertificateToCertify(d->target);
     if (d->uids.size()) {
         d->dialog->setSelectedUserIDs(d->uids);
     }
@@ -216,7 +239,7 @@ void CertifyCertificateCommand::Private::slotDialogRejected()
 void CertifyCertificateCommand::Private::slotResult(const Error &err)
 {
     if (!err && !err.isCanceled() && dialog && dialog->exportableCertificationSelected() && dialog->sendToServer()) {
-        auto const cmd = new ExportOpenPGPCertsToServerCommand(key());
+        auto const cmd = new ExportOpenPGPCertsToServerCommand(target);
         cmd->start();
     } else if (!err) {
         information(i18n("Certification successful."),
@@ -224,7 +247,7 @@ void CertifyCertificateCommand::Private::slotResult(const Error &err)
     } else {
         error(i18n("<p>An error occurred while trying to certify<br/><br/>"
                    "<b>%1</b>:</p><p>\t%2</p>",
-              Formatting::formatForComboBox(key()),
+              Formatting::formatForComboBox(target),
               QString::fromUtf8(err.asString())),
               i18n("Certification Error"));
     }
@@ -239,11 +262,24 @@ void CertifyCertificateCommand::Private::slotCertificationPrepared()
 {
     Q_ASSERT(dialog);
 
+    const auto selectedUserIds = dialog->selectedUserIDs();
+    std::vector<unsigned int> userIdIndexes;
+    userIdIndexes.reserve(selectedUserIds.size());
+    for (unsigned int i = 0, numUserIds = target.numUserIDs(); i < numUserIds; ++i) {
+        const auto userId = target.userID(i);
+        const bool userIdIsSelected = Kleo::any_of(selectedUserIds, [userId](const auto &uid) {
+            return Kleo::userIDsAreEqual(userId, uid);
+        });
+        if (userIdIsSelected) {
+            userIdIndexes.push_back(i);
+        }
+    }
+
     createJob();
     Q_ASSERT(job);
     job->setExportable(dialog->exportableCertificationSelected());
     job->setNonRevocable(dialog->nonRevocableCertificationSelected());
-    job->setUserIDsToSign(dialog->selectedUserIDs());
+    job->setUserIDsToSign(userIdIndexes);
     job->setSigningKey(dialog->selectedSecretKey());
     job->setCheckLevel(dialog->selectedCheckLevel());
     if (!dialog->tags().isEmpty()) {
@@ -259,7 +295,7 @@ void CertifyCertificateCommand::Private::slotCertificationPrepared()
         job->setExpirationDate(dialog->expirationDate());
     }
 
-    if (const Error err = job->start(key())) {
+    if (const Error err = job->start(target)) {
         slotResult(err);
     }
 }
@@ -289,7 +325,7 @@ void CertifyCertificateCommand::Private::createJob()
 {
     Q_ASSERT(!job);
 
-    Q_ASSERT(key().protocol() == OpenPGP);
+    Q_ASSERT(target.protocol() == OpenPGP);
     const auto backend = QGpgME::openpgp();
     if (!backend) {
         return;
