@@ -166,31 +166,23 @@ void Command::setView(QAbstractItemView *view)
         return;
     }
     d->view_ = view;
-    if (!view || !d->indexes_.empty()) {
+    if (!view || !d->keys_.empty()) {
+        return;
+    }
+    const auto *const keyListModel = dynamic_cast<KeyListModelInterface *>(view->model());
+    if (!keyListModel) {
+        qCWarning(KLEOPATRA_LOG) << "view" << view << "has not key list model";
         return;
     }
     const QItemSelectionModel *const sm = view->selectionModel();
     if (!sm) {
-        qCWarning(KLEOPATRA_LOG) << "view " << (void *)view << " has no selectionModel!";
+        qCWarning(KLEOPATRA_LOG) << "view" << view << "has no selection model";
         return;
     }
     const QList<QModelIndex> selected = sm->selectedRows();
-    if (!selected.empty()) {
-        std::copy(selected.begin(), selected.end(), std::back_inserter(d->indexes_));
-        return;
-    }
-}
-
-void Command::setIndex(const QModelIndex &idx)
-{
-    d->indexes_.clear();
-    d->indexes_.push_back(idx);
-}
-
-void Command::setIndexes(const QList<QModelIndex> &idx)
-{
-    d->indexes_.clear();
-    std::copy(idx.begin(), idx.end(), std::back_inserter(d->indexes_));
+    std::transform(selected.begin(), selected.end(), std::back_inserter(d->keys_), [keyListModel](const auto &idx) {
+        return keyListModel->key(idx);
+    });
 }
 
 void Command::setKey(const Key &key)
@@ -308,6 +300,6 @@ Command *Command::commandForQuery(const QString &query)
     if (key.isNull()) {
         return new LookupCertificatesCommand(query, nullptr);
     } else {
-        return new DetailsCommand(key, nullptr);
+        return new DetailsCommand(key);
     }
 }
