@@ -17,6 +17,7 @@
 #include "importcertificatescommand_p.h"
 
 #include "certifycertificatecommand.h"
+#include <utils/keys.h>
 #include <settings.h>
 #include "kleopatra_debug.h"
 
@@ -30,7 +31,6 @@
 #include <Libkleo/Formatting>
 #include <Libkleo/Stl_Util>
 
-#include <QGpgME/KeyListJob>
 #include <QGpgME/Protocol>
 #include <QGpgME/ImportJob>
 #include <QGpgME/ImportFromKeyserverJob>
@@ -47,6 +47,7 @@
 
 #include <KLocalizedString>
 #include <KMessageBox>
+#include <kwidgetsaddons_version.h>
 
 #include <QByteArray>
 #include <QEventLoop>
@@ -349,6 +350,11 @@ static QString make_report(const std::vector<ImportResultData> &results,
     return lines.join(QLatin1String{});
 }
 
+static bool isImportFromSingleSource(const std::vector<ImportResultData> &res)
+{
+    return (res.size() == 1) || (res.size() == 2 && res[0].id == res[1].id);
+}
+
 static QString make_message_report(const std::vector<ImportResultData> &res,
                                    const std::vector<ImportedGroup> &groups)
 {
@@ -356,8 +362,7 @@ static QString make_message_report(const std::vector<ImportResultData> &res,
     if (res.empty()) {
         report += i18n("No imports (should not happen, please report a bug).");
     } else {
-        const bool singleSource = (res.size() == 1) || (res.size() == 2 && res[0].id == res[1].id);
-        const QString title = singleSource && !res.front().id.isEmpty() ?
+        const QString title = isImportFromSingleSource(res) && !res.front().id.isEmpty() ?
                               i18n("Detailed results of importing %1:", res.front().id) :
                               i18n("Detailed results of import:");
         report += QLatin1String{"<p>"} + title + QLatin1String{"</p>"};
@@ -395,6 +400,10 @@ bool ImportCertificatesCommand::Private::showPleaseCertify(const GpgME::Import &
         // No such key most likely not OpenPGP
         return false;
     }
+    if (!Kleo::canBeCertified(key)) {
+        // key is expired or revoked
+        return false;
+    }
 
     for (const auto &uid: key.userIDs()) {
         if (uid.validity() >= GpgME::UserID::Marginal) {
@@ -407,16 +416,30 @@ bool ImportCertificatesCommand::Private::showPleaseCertify(const GpgME::Import &
         << i18n("Using a business card.")
         << i18n("Confirming it on a trusted website.");
 
+#if KWIDGETSADDONS_VERSION >= QT_VERSION_CHECK(5, 100, 0)
+    auto sel = KMessageBox::questionTwoActions(parentWidgetOrView(),
+#else
     auto sel = KMessageBox::questionYesNo(parentWidgetOrView(),
-                i18n("In order to mark the certificate as valid (green) it needs to be certified.") + QStringLiteral("<br>") +
-                i18n("Certifying means that you check the Fingerprint.") + QStringLiteral("<br>") +
-                i18n("Some suggestions to do this are:") +
-                QStringLiteral("<li><ul>%1").arg(suggestions.join(QStringLiteral("</ul><ul>"))) +
-                QStringLiteral("</ul></li>") +
-                i18n("Do you wish to start this process now?"),
-                i18nc("@title", "You have imported a new certificate (public key)"),
-                KStandardGuiItem::yes(), KStandardGuiItem::no(), QStringLiteral("CertifyQuestion"));
+#endif
+                                               i18n("In order to mark the certificate as valid (green) it needs to be certified.") + QStringLiteral("<br>")
+                                                   + i18n("Certifying means that you check the Fingerprint.") + QStringLiteral("<br>")
+                                                   + i18n("Some suggestions to do this are:")
+                                                   + QStringLiteral("<li><ul>%1").arg(suggestions.join(QStringLiteral("</ul><ul>")))
+                                                   + QStringLiteral("</ul></li>") + i18n("Do you wish to start this process now?"),
+                                               i18nc("@title", "You have imported a new certificate (public key)"),
+#if KWIDGETSADDONS_VERSION >= QT_VERSION_CHECK(5, 100, 0)
+                                               KGuiItem(i18n("Import")),
+                                               KStandardGuiItem::cancel(),
+#else
+                                          KStandardGuiItem::yes(),
+                                          KStandardGuiItem::no(),
+#endif
+                                               QStringLiteral("CertifyQuestion"));
+#if KWIDGETSADDONS_VERSION >= QT_VERSION_CHECK(5, 100, 0)
+    if (sel == KMessageBox::ButtonCode::PrimaryAction) {
+#else
     if (sel == KMessageBox::Yes) {
+#endif
         QEventLoop loop;
         auto cmd = new Commands::CertifyCertificateCommand(key);
         cmd->setParentWidget(parentWidgetOrView());
@@ -427,11 +450,39 @@ bool ImportCertificatesCommand::Private::showPleaseCertify(const GpgME::Import &
     return true;
 }
 
+namespace
+{
+/**
+ * Returns the Import of an OpenPGP key, if a single certificate was imported and this was an OpenPGP key.
+ * Otherwise, returns a null Import.
+ */
+auto getSingleOpenPGPImport(const std::vector<ImportResultData> &res)
+{
+    static const Import nullImport;
+    if (!isImportFromSingleSource(res)) {
+        return nullImport;
+    }
+    const auto numImported = std::accumulate(res.cbegin(), res.cend(), 0, [](auto s, const auto &r) {
+        return s + r.result.numImported();
+    });
+    if (numImported > 1) {
+        return nullImport;
+    }
+    if ((res.size() >= 1) && (res[0].protocol == GpgME::OpenPGP) && (res[0].result.numImported() == 1) && (res[0].result.imports().size() == 1)) {
+        return res[0].result.imports()[0];
+    } else if ((res.size() == 2) && (res[1].protocol == GpgME::OpenPGP) && (res[1].result.numImported() == 1) && (res[1].result.imports().size() == 1)) {
+        return res[1].result.imports()[0];
+    }
+    return nullImport;
+}
+}
+
 void ImportCertificatesCommand::Private::showDetails(const std::vector<ImportResultData> &res,
                                                      const std::vector<ImportedGroup> &groups)
 {
-    if (res.size() == 1 && res[0].result.numImported() == 1 && res[0].result.imports().size() == 1) {
-        if (showPleaseCertify(res[0].result.imports()[0])) {
+    const auto singleOpenPGPImport = getSingleOpenPGPImport(res);
+    if (!singleOpenPGPImport.isNull()) {
+        if (showPleaseCertify(singleOpenPGPImport)) {
             return;
         }
     }
@@ -532,12 +583,12 @@ static void handleOwnerTrust(const std::vector<ImportResultData> &results)
                 return;
             }
 
-            QStringList uids;
             const auto toTrustOwnerUserIDs{toTrustOwner.userIDs()};
-            uids.reserve(toTrustOwnerUserIDs.size());
-            for (const UserID &uid : toTrustOwnerUserIDs) {
-                uids << Formatting::prettyNameAndEMail(uid);
-            }
+            // ki18n(" ") as initializer because initializing with empty string leads to
+            // (I18N_EMPTY_MESSAGE)
+            const KLocalizedString uids = std::accumulate(toTrustOwnerUserIDs.cbegin(), toTrustOwnerUserIDs.cend(), KLocalizedString{ki18n(" ")}, [](KLocalizedString temp, const auto &uid) {
+                return kxi18nc("@info", "%1<item>%2</item>").subs(temp).subs(Formatting::prettyNameAndEMail(uid));
+            });
 
             const QString str = xi18nc("@info",
                 "<title>You have imported a Secret Key.</title>"
@@ -545,16 +596,28 @@ static void handleOwnerTrust(const std::vector<ImportResultData> &results)
                 "<numid>%1</numid>"
                 "</para>"
                 "<para>And claims the user IDs:"
-                "<list><item>%2</item></list>"
+                "<list>%2</list>"
                 "</para>"
                 "Is this your own key? (Set trust level to ultimate)",
                 QString::fromUtf8(fingerPr),
-                uids.join(QLatin1String("</item><item>")));
+                uids);
 
-            int k = KMessageBox::questionYesNo(nullptr, str, i18nc("@title:window",
-                                                               "Secret key imported"));
+#if KWIDGETSADDONS_VERSION >= QT_VERSION_CHECK(5, 100, 0)
+            int k = KMessageBox::questionTwoActions(nullptr,
+                                                    str,
+#else
+            int k = KMessageBox::questionYesNo(nullptr,
+                                               str,
+#endif
+                                                    i18nc("@title:window", "Secret key imported"),
+                                                    KGuiItem(i18n("Import")),
+                                                    KStandardGuiItem::cancel());
 
+#if KWIDGETSADDONS_VERSION >= QT_VERSION_CHECK(5, 100, 0)
+            if (k == KMessageBox::ButtonCode::PrimaryAction) {
+#else
             if (k == KMessageBox::Yes) {
+#endif
                 //To use the ChangeOwnerTrustJob over
                 //the CryptoBackendFactory
                 const QGpgME::Protocol *const backend = QGpgME::openpgp();
@@ -904,8 +967,9 @@ void ImportCertificatesCommand::Private::startImport(GpgME::Protocol protocol, [
     keyCacheAutoRefreshSuspension = KeyCache::mutableInstance()->suspendAutoRefresh();
 
     std::vector<QMetaObject::Connection> connections = {
-        connect(job.get(), SIGNAL(result(GpgME::ImportResult)),
-                q, SLOT(importResult(GpgME::ImportResult))),
+        connect(job.get(), &AbstractImportJob::result, q, [this](const GpgME::ImportResult &result) {
+            importResult(result);
+        }),
         connect(job.get(), &Job::progress,
                 q, &Command::progress)
     };
@@ -1003,6 +1067,5 @@ void ImportCertificatesCommand::doCancel()
 #undef d
 #undef q
 
-#include "moc_importcertificatescommand.cpp"
 #include "importcertificatescommand.moc"
-
+#include "moc_importcertificatescommand.cpp"
