@@ -29,7 +29,7 @@
 #include <Libkleo/Formatting>
 #include <Libkleo/Stl_Util>
 
-#include <QGpgME/CryptoConfig>
+#include <QGpgME/Debug>
 #include <QGpgME/Protocol>
 #include <QGpgME/KeyListJob>
 #include <QGpgME/ImportFromKeyserverJob>
@@ -402,7 +402,9 @@ void LookupCertificatesCommand::Private::slotWKDLookupResult(const WKDLookupResu
         qCDebug(KLEOPATRA_LOG) << __func__ << "unknown sender()" << q->sender();
     }
 
-    keyListing.result.mergeWith(KeyListResult{result.error()});
+    // we do not want to bother the user with errors during the WKD lookup;
+    // therefore, we log the result, but we do not merge it into keyListing.result
+    qCDebug(KLEOPATRA_LOG) << "Result of WKD lookup:" << result.error();
 
     const auto keys = removeKeysNotMatchingEmail(result.keyData().toKeys(GpgME::OpenPGP), result.pattern());
     if (!keys.empty()) {
@@ -546,7 +548,7 @@ void LookupCertificatesCommand::Private::slotSaveAsRequested(const std::vector<K
 
 void LookupCertificatesCommand::Private::slotDetailsRequested(const Key &key)
 {
-    Command *const cmd = new DetailsCommand(key, view(), controller());
+    Command *const cmd = new DetailsCommand(key);
     cmd->setParentWidget(dialogOrParentWidgetOrView());
     cmd->start();
 }
