@@ -18,7 +18,6 @@
 
 #include <settings.h>
 
-#include <QGpgME/Protocol>
 #include <Libkleo/KeyFilterManager>
 #include <Libkleo/Dn>
 #include <Libkleo/DNAttributeOrderConfigWidget>
@@ -467,7 +466,7 @@ void AppearanceConfigWidget::load()
 {
     if (d->dnOrderWidget) {
         const Settings settings;
-        d->dnOrderWidget->setAttributeOrder(settings.attributeOrder());
+        d->dnOrderWidget->setAttributeOrder(DN::attributeOrder());
         d->dnOrderWidget->setEnabled(!settings.isImmutable(QStringLiteral("AttributeOrder")));
     }
 
