@@ -12,8 +12,7 @@
     SPDX-License-Identifier: GPL-2.0-only
 */
 
-#ifndef __KLEOPATRA_CONF_CONFIGUREDIALOG_H__
-#define __KLEOPATRA_CONF_CONFIGUREDIALOG_H__
+#pragma once
 
 #include <config-kleopatra.h>
 
@@ -23,13 +22,9 @@
  * and the KleoPageConfigDialog. Which is just a KPageView
  * with the necessary bits of the KCMultiDialog behavior.
  */
-#if HAVE_KCMUTILS
-# include <KCMultiDialog>
-class ConfigureDialog : public KCMultiDialog
-#else
 # include "kleopageconfigdialog.h"
+
 class ConfigureDialog : public KleoPageConfigDialog
-#endif
 {
     Q_OBJECT
 public:
@@ -41,4 +36,3 @@ protected:
 
 };
 
-#endif /* __KLEOPATRA_CONF_CONFIGUREDIALOG_H__ */
