@@ -9,6 +9,8 @@
 
 #pragma once
 
+#include "labelledwidget.h"
+
 #include <QWidget>
 
 #include <utils/pimpl_ptr.h>
@@ -20,6 +22,10 @@ class QPushButton;
 
 namespace Kleo
 {
+class EMailOperationsPreferences;
+class FileOperationsPreferences;
+class Settings;
+
 namespace Config
 {
 
@@ -43,17 +49,22 @@ private:
     void setupProfileGui(QBoxLayout *layout);
     void applyProfile(const QString &profile);
 
+    void load(const Kleo::EMailOperationsPreferences &emailPrefs,
+              const Kleo::FileOperationsPreferences &filePrefs,
+              const Kleo::Settings &settings);
+
 private:
-    QCheckBox *mQuickEncryptCB,
-              *mQuickSignCB,
-              *mPGPFileExtCB,
-              *mAutoDecryptVerifyCB,
-              *mASCIIArmorCB,
-              *mTmpDirCB,
-              *mSymmetricOnlyCB;
-    QComboBox *mChecksumDefinitionCB,
-              *mArchiveDefinitionCB;
-    QPushButton *mApplyBtn;
+    QCheckBox *mQuickEncryptCB = nullptr;
+    QCheckBox *mQuickSignCB = nullptr;
+    QCheckBox *mPGPFileExtCB = nullptr;
+    QCheckBox *mAutoDecryptVerifyCB = nullptr;
+    QCheckBox *mAutoExtractArchivesCB = nullptr;
+    QCheckBox *mASCIIArmorCB = nullptr;
+    QCheckBox *mTmpDirCB = nullptr;
+    QCheckBox *mSymmetricOnlyCB = nullptr;
+    Kleo::LabelledWidget<QComboBox> mChecksumDefinitionCB;
+    Kleo::LabelledWidget<QComboBox> mArchiveDefinitionCB;
+    QPushButton *mApplyBtn = nullptr;
 };
 
 }
