@@ -170,8 +170,13 @@ DirectoryServicesConfigurationPage::Private::Private(DirectoryServicesConfigurat
     mMaxItems.createWidgets(q);
     mMaxItems.label()->setText(i18n("&Maximum number of items returned by query:"));
     mMaxItems.widget()->setMinimum(0);
+#if QT_DEPRECATED_SINCE(5, 14)
+    connect(mMaxItems.widget(), qOverload<int>(&QSpinBox::valueChanged),
+            q, &DirectoryServicesConfigurationPage::markAsChanged);
+#else
     connect(mMaxItems.widget(), &QSpinBox::valueChanged,
             q, &DirectoryServicesConfigurationPage::markAsChanged);
+#endif
     glay->addWidget(mMaxItems.label(), row, 0);
     glay->addWidget(mMaxItems.widget(), row, 1);
 
@@ -183,7 +188,7 @@ DirectoryServicesConfigurationPage::Private::Private(DirectoryServicesConfigurat
     mFetchMissingSignerKeysCB->setToolTip(
         xi18nc("@info:tooltip",
                "If enabled, then Kleopatra will automatically try to retrieve the keys "
-               "that were used to certify the user ids of newly imported OpenPGP keys."));
+               "that were used to certify the user IDs of newly imported OpenPGP keys."));
     connect(mFetchMissingSignerKeysCB, &QCheckBox::toggled,
             q, &DirectoryServicesConfigurationPage::markAsChanged);
     glay->addWidget(mFetchMissingSignerKeysCB, row, 0, 1, 3);
