@@ -20,7 +20,6 @@
 #include <Libkleo/Compat>
 
 #include <QGpgME/CryptoConfig>
-#include <QGpgME/Protocol>
 
 #include <KLocalizedString>
 #include "kleopatra_debug.h"
@@ -40,15 +39,46 @@ class SMimeValidationConfigurationWidget::Private
 public:
     explicit Private(SMimeValidationConfigurationWidget *qq)
         : q(qq),
-          customHTTPProxyWritable(false),
-          ui(q)
+          ui(qq)
     {
 #if HAVE_QDBUS
         QDBusConnection::sessionBus().connect(QString(), QString(), QStringLiteral("org.kde.kleo.CryptoConfig"), QStringLiteral("changed"), q, SLOT(load()));
 #endif
+        auto changedSignal = &SMimeValidationConfigurationWidget::changed;
+        connect(ui.intervalRefreshCB, &QCheckBox::toggled, q, changedSignal);
+#if QT_DEPRECATED_SINCE(5, 14)
+        connect(ui.intervalRefreshSB, qOverload<int>(&QSpinBox::valueChanged), q, changedSignal);
+#else
+        connect(ui.intervalRefreshSB, &QSpinBox::valueChanged, q, changedSignal);
+#endif
+        connect(ui.OCSPCB, &QCheckBox::toggled, q, changedSignal);
+        connect(ui.OCSPResponderURL, &QLineEdit::textChanged, q, changedSignal);
+
+        auto certRequesterSignal = &KleopatraClientCopy::Gui::CertificateRequester::selectedCertificatesChanged;
+        connect(ui.OCSPResponderSignature, certRequesterSignal, q, changedSignal);
+
+        connect(ui.doNotCheckCertPolicyCB, &QCheckBox::toggled, q, changedSignal);
+        connect(ui.neverConsultCB, &QCheckBox::toggled, q, changedSignal);
+        connect(ui.allowMarkTrustedCB, &QCheckBox::toggled, q, changedSignal);
+        connect(ui.fetchMissingCB, &QCheckBox::toggled, q, changedSignal);
+        connect(ui.ignoreServiceURLCB, &QCheckBox::toggled, q, changedSignal);
+        connect(ui.ignoreHTTPDPCB, &QCheckBox::toggled, q, changedSignal);
+        connect(ui.disableHTTPCB, &QCheckBox::toggled, q, changedSignal);
+        connect(ui.honorHTTPProxyRB, &QRadioButton::toggled, q, changedSignal);
+        connect(ui.useCustomHTTPProxyRB, &QRadioButton::toggled, q, changedSignal);
+        connect(ui.customHTTPProxy, &QLineEdit::textChanged, q, changedSignal);
+        connect(ui.ignoreLDAPDPCB, &QCheckBox::toggled, q, changedSignal);
+        connect(ui.disableLDAPCB, &QCheckBox::toggled, q, changedSignal);
+        connect(ui.customLDAPProxy, &QLineEdit::textChanged, q, changedSignal);
+
+        auto enableDisableSlot = [this]() {
+            enableDisableActions();
+        };
+        connect(ui.useCustomHTTPProxyRB, &QRadioButton::toggled, q, enableDisableSlot);
+        connect(ui.disableHTTPCB, &QCheckBox::toggled, q, enableDisableSlot);
     }
 
-    bool customHTTPProxyWritable;
+    bool customHTTPProxyWritable = false;
 
 private:
     void enableDisableActions()
@@ -75,37 +105,6 @@ private:
                 l->setContentsMargins(0, 0, 0, 0);
             }
 
-            const struct {
-                QObject *object;
-                const char *signal;
-            } sources[] = {
-                { intervalRefreshCB, SIGNAL(toggled(bool)) },
-                { intervalRefreshSB, SIGNAL(valueChanged(int)) },
-                { OCSPCB, SIGNAL(toggled(bool)) },
-                { OCSPResponderURL, SIGNAL(textChanged(QString)) },
-                { OCSPResponderSignature, SIGNAL(selectedCertificatesChanged(QStringList)) },
-                { doNotCheckCertPolicyCB, SIGNAL(toggled(bool)) },
-                { neverConsultCB, SIGNAL(toggled(bool)) },
-                { allowMarkTrustedCB, SIGNAL(toggled(bool)) },
-                { fetchMissingCB, SIGNAL(toggled(bool)) },
-                { ignoreServiceURLCB, SIGNAL(toggled(bool)) },
-                { ignoreHTTPDPCB, SIGNAL(toggled(bool)) },
-                { disableHTTPCB, SIGNAL(toggled(bool)) },
-                { honorHTTPProxyRB, SIGNAL(toggled(bool)) },
-                { useCustomHTTPProxyRB, SIGNAL(toggled(bool)) },
-                { customHTTPProxy, SIGNAL(textChanged(QString)) },
-                { ignoreLDAPDPCB, SIGNAL(toggled(bool)) },
-                { disableLDAPCB, SIGNAL(toggled(bool)) },
-                { customLDAPProxy, SIGNAL(textChanged(QString)) },
-            };
-            for (unsigned int i = 0; i < sizeof sources / sizeof * sources; ++i) {
-                connect(sources[i].object, sources[i].signal, q, SIGNAL(changed()));
-            }
-            connect(useCustomHTTPProxyRB, SIGNAL(toggled(bool)),
-                    q, SLOT(enableDisableActions()));
-            connect(disableHTTPCB, SIGNAL(toggled(bool)),
-                    q, SLOT(enableDisableActions()));
-
             OCSPResponderSignature->setOnlyX509CertificatesAllowed(true);
             OCSPResponderSignature->setOnlySigningCertificatesAllowed(true);
             OCSPResponderSignature->setMultipleCertificatesAllowed(false);
