@@ -10,8 +10,7 @@
     SPDX-License-Identifier: GPL-2.0-or-later
 */
 
-#ifndef __KLEOPATRA_CRYPTO_AUTODECRYPTVERIFYFILESCONTROLLER_H__
-#define __KLEOPATRA_CRYPTO_AUTODECRYPTVERIFYFILESCONTROLLER_H__
+#pragma once
 
 #include "crypto/decryptverifyfilescontroller.h"
 
@@ -33,7 +32,7 @@ public:
     explicit AutoDecryptVerifyFilesController(QObject *parent = nullptr);
     explicit AutoDecryptVerifyFilesController(const std::shared_ptr<const ExecutionContext> &ctx, QObject *parent = nullptr);
 
-    ~AutoDecryptVerifyFilesController();
+    ~AutoDecryptVerifyFilesController() override;
 
     void setFiles(const QStringList &files) override;
     void setOperation(DecryptVerifyOperation op) override;
@@ -56,4 +55,3 @@ private:
 }
 }
 
-#endif // __KLEOPATRA_CRYPTO_AUTODECRYPTVERIFYFILESCONTROLLER_H__
