@@ -45,7 +45,7 @@ public:
 
     QString overview() const override;
     QString details() const override;
-    int errorCode() const override;
+    GpgME::Error error() const override;
     QString errorString() const override;
     VisualCode code() const override;
     AuditLog auditLog() const override;
@@ -186,7 +186,7 @@ std::unique_ptr<QGpgME::EncryptJob> EncryptEMailTask::Private::createJob(GpgME::
 
 void EncryptEMailTask::Private::slotResult(const EncryptionResult &result)
 {
-    const QGpgME::Job *const job = qobject_cast<const QGpgME::Job *>(q->sender());
+    const auto *const job = qobject_cast<const QGpgME::Job *>(q->sender());
     if (result.error().code()) {
         output->cancel();
     } else {
@@ -205,9 +205,9 @@ QString EncryptEMailResult::details() const
     return QString();
 }
 
-int EncryptEMailResult::errorCode() const
+GpgME::Error EncryptEMailResult::error() const
 {
-    return m_result.error().encodedError();
+    return m_result.error();
 }
 
 QString EncryptEMailResult::errorString() const
