@@ -28,6 +28,7 @@
 #include <KMessageWidget>
 
 #include "kleopatra_debug.h"
+#include <Libkleo/Compliance>
 #include <Libkleo/GnuPG>
 #include <Libkleo/Formatting>
 #include <Libkleo/SystemInfo>
@@ -40,6 +41,7 @@
 #include <QLabel>
 #include <QIcon>
 #include <QCheckBox>
+#include <QPushButton>
 #include <QStyle>
 
 #include <gpgme++/key.h>
@@ -189,13 +191,13 @@ public:
         auto messageWidget = new KMessageWidget;
         messageWidget->setMessageType(KMessageWidget::Error);
         messageWidget->setIcon(style()->standardIcon(QStyle::SP_MessageBoxCritical, nullptr, this));
-        messageWidget->setText(i18n("Signing and encrypting files is not possible."));
+        messageWidget->setText(i18n("Invalid compliance settings for signing and encrypting files."));
         messageWidget->setToolTip(xi18nc("@info %1 is a placeholder for the name of a compliance mode. E.g. NATO RESTRICTED compliant or VS-NfD compliant",
                                          "<para>You cannot use <application>Kleopatra</application> for signing or encrypting files "
                                          "because the <application>GnuPG</application> system used by <application>Kleopatra</application> is not %1.</para>",
-                                         Formatting::deVsString()));
+                                         DeVSCompliance::name(true)));
         messageWidget->setCloseButtonVisible(false);
-        messageWidget->setVisible(Kleo::gnupgUsesDeVsCompliance() && !Kleo::gnupgIsDeVsCompliant());
+        messageWidget->setVisible(DeVSCompliance::isActive() && !DeVSCompliance::isCompliant());
         vLay->addWidget(messageWidget);
 
         setMinimumHeight(300);
@@ -213,7 +215,7 @@ public:
 
     bool isComplete() const override
     {
-        if (Kleo::gnupgUsesDeVsCompliance() && !Kleo::gnupgIsDeVsCompliant()) {
+        if (DeVSCompliance::isActive() && !DeVSCompliance::isCompliant()) {
             return false;
         }
         return mWidget->isComplete();
@@ -255,12 +257,12 @@ public:
 
     bool validatePage() override
     {
-        if (Kleo::gnupgUsesDeVsCompliance() && !Kleo::gnupgIsDeVsCompliant()) {
-            KMessageBox::sorry(topLevelWidget(),
+        if (DeVSCompliance::isActive() && !DeVSCompliance::isCompliant()) {
+            KMessageBox::error(topLevelWidget(),
                                xi18nc("@info %1 is a placeholder for the name of a compliance mode. E.g. NATO RESTRICTED compliant or VS-NfD compliant",
                                       "<para>Sorry! You cannot use <application>Kleopatra</application> for signing or encrypting files "
                                       "because the <application>GnuPG</application> system used by <application>Kleopatra</application> is not %1.</para>",
-                                      Formatting::deVsString()));
+                                      DeVSCompliance::name(true)));
             return false;
         }
         bool sign = !mWidget->signKey().isNull();
@@ -436,29 +438,32 @@ public:
     }
 
 private Q_SLOTS:
-    void updateCommitButton(const QString &label)
+    void updateCommitButton(const SignEncryptWidget::Operation op)
     {
         if (mParent->currentPage() != this) {
             return;
         }
-        auto btn = mParent->button(QWizard::CommitButton);
+        QString label;
+        switch (op) {
+        case SignEncryptWidget::Sign:
+            label = i18nc("@action:button", "Sign");
+            break;
+        case SignEncryptWidget::Encrypt:
+            label = i18nc("@action:button", "Encrypt");
+            break;
+        case SignEncryptWidget::SignAndEncrypt:
+            label = i18nc("@action:button", "Sign / Encrypt");
+            break;
+        default:
+            ;
+        };
+        auto btn = qobject_cast<QPushButton*>(mParent->button(QWizard::CommitButton));
         if (!label.isEmpty()) {
             mParent->setButtonText(QWizard::CommitButton, label);
-            if (Kleo::gnupgUsesDeVsCompliance()) {
-                const bool de_vs = Kleo::gnupgIsDeVsCompliant() && mWidget->isDeVsAndValid();
-                btn->setIcon(QIcon::fromTheme(de_vs
-                                             ? QStringLiteral("security-high")
-                                             : QStringLiteral("security-medium")));
-                if (!SystemInfo::isHighContrastModeActive()) {
-                    btn->setStyleSheet(QStringLiteral("QPushButton { background-color: %1; }").arg(de_vs
-                                    ? KColorScheme(QPalette::Active, KColorScheme::View).background(KColorScheme::PositiveBackground).color().name()
-                                    : KColorScheme(QPalette::Active, KColorScheme::View).background(KColorScheme::NegativeBackground).color().name()));
-                }
-                mParent->setLabelText(de_vs
-                        ? i18nc("%1 is a placeholder for the name of a compliance mode. E.g. NATO RESTRICTED compliant or VS-NfD compliant",
-                            "%1 communication possible.", Formatting::deVsString())
-                        : i18nc("%1 is a placeholder for the name of a compliance mode. E.g. NATO RESTRICTED compliant or VS-NfD compliant",
-                            "%1 communication not possible.", Formatting::deVsString()));
+            if (DeVSCompliance::isActive()) {
+                const bool de_vs = DeVSCompliance::isCompliant() && mWidget->isDeVsAndValid();
+                DeVSCompliance::decorate(btn, de_vs);
+                mParent->setLabelText(DeVSCompliance::name(de_vs));
             }
         } else {
             mParent->setButtonText(QWizard::CommitButton, i18n("Next"));
@@ -546,7 +551,7 @@ SignEncryptFilesWizard::SignEncryptFilesWizard(QWidget *parent, Qt::WindowFlags
 {
     readConfig();
 
-    const bool de_vs = Kleo::gnupgUsesDeVsCompliance();
+    const bool de_vs = DeVSCompliance::isActive();
 #ifdef Q_OS_WIN
     // Enforce modern style to avoid vista style ugliness.
     setWizardStyle(QWizard::ModernStyle);
