@@ -2,32 +2,9 @@
     crypto/task.cpp
 
     This file is part of Kleopatra, the KDE keymanager
-    Copyright (c) 2007 Klarälvdalens Datakonsult AB
+    SPDX-FileCopyrightText: 2007 Klarälvdalens Datakonsult AB
 
-    Kleopatra is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    Kleopatra is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
-
-    In addition, as a special exception, the copyright holders give
-    permission to link the code of this program with any edition of
-    the Qt library by Trolltech AS, Norway (or with modified versions
-    of Qt that use the same license as Qt), and distribute linked
-    combinations including the two.  You must obey the GNU General
-    Public License in all respects for all of the code used other than
-    Qt.  If you modify this file, you may extend this exception to
-    your version of the file, but you are not obligated to do so.  If
-    you do not wish to do so, delete this exception statement from
-    your version.
+    SPDX-License-Identifier: GPL-2.0-or-later
 */
 
 #include <config-kleopatra.h>
@@ -35,9 +12,9 @@
 #include "task_p.h"
 #include "kleopatra_debug.h"
 
-#include <Libkleo/Exception>
+#include <Libkleo/KleoException>
 
-#include <utils/gnupg-helper.h>
+#include <Libkleo/GnuPG>
 #include <utils/auditlog.h>
 
 #include <gpgme++/exception.h>
@@ -47,7 +24,6 @@
 #include <KIconLoader>
 #include <KLocalizedString>
 
-#include <QString>
 
 using namespace Kleo;
 using namespace Kleo::Crypto;
@@ -59,8 +35,8 @@ namespace
 class ErrorResult : public Task::Result
 {
 public:
-    ErrorResult(int code, const QString &details)
-        : Task::Result(), m_code(code), m_details(details) {}
+    ErrorResult(const GpgME::Error &error, const QString &details)
+        : Task::Result(), m_error(error), m_details(details) {}
 
     QString overview() const override
     {
@@ -70,9 +46,9 @@ public:
     {
         return QString();
     }
-    int errorCode() const override
+    GpgME::Error error() const override
     {
-        return m_code;
+        return m_error;
     }
     QString errorString() const override
     {
@@ -87,8 +63,8 @@ public:
         return AuditLog();
     }
 private:
-    int m_code;
-    QString m_details;
+    const GpgME::Error m_error;
+    const QString m_details;
 };
 }
 
@@ -136,10 +112,10 @@ bool Task::asciiArmor() const
     return d->m_asciiArmor;
 }
 
-std::shared_ptr<Task> Task::makeErrorTask(int code, const QString &details, const QString &label)
+std::shared_ptr<Task> Task::makeErrorTask(const GpgME::Error &error, const QString &details, const QString &label)
 {
     const std::shared_ptr<SimpleTask> t(new SimpleTask(label));
-    t->setResult(t->makeErrorResult(code, details));
+    t->setResult(t->makeErrorResult(error, details));
     return t;
 }
 
@@ -181,20 +157,20 @@ void Task::start()
     try {
         doStart();
     } catch (const Kleo::Exception &e) {
-        QMetaObject::invokeMethod(this, "emitError", Qt::QueuedConnection, Q_ARG(int, e.error().encodedError()), Q_ARG(QString, e.message()));
+        QMetaObject::invokeMethod(this, "emitError", Qt::QueuedConnection, Q_ARG(GpgME::Error, e.error()), Q_ARG(QString, e.message()));
     } catch (const GpgME::Exception &e) {
-        QMetaObject::invokeMethod(this, "emitError", Qt::QueuedConnection, Q_ARG(int, e.error().encodedError()), Q_ARG(QString, QString::fromLocal8Bit(e.what())));
+        QMetaObject::invokeMethod(this, "emitError", Qt::QueuedConnection, Q_ARG(GpgME::Error, e.error()), Q_ARG(QString, QString::fromLocal8Bit(e.what())));
     } catch (const std::exception &e) {
-        QMetaObject::invokeMethod(this, "emitError", Qt::QueuedConnection, Q_ARG(int, makeGnuPGError(GPG_ERR_UNEXPECTED)), Q_ARG(QString, QString::fromLocal8Bit(e.what())));
+        QMetaObject::invokeMethod(this, "emitError", Qt::QueuedConnection, Q_ARG(GpgME::Error, Error::fromCode(GPG_ERR_UNEXPECTED)), Q_ARG(QString, QString::fromLocal8Bit(e.what())));
     } catch (...) {
-        QMetaObject::invokeMethod(this, "emitError", Qt::QueuedConnection, Q_ARG(int, makeGnuPGError(GPG_ERR_UNEXPECTED)), Q_ARG(QString, i18n("Unknown exception in Task::start()")));
+        QMetaObject::invokeMethod(this, "emitError", Qt::QueuedConnection, Q_ARG(GpgME::Error, Error::fromCode(GPG_ERR_UNEXPECTED)), Q_ARG(QString, i18n("Unknown exception in Task::start()")));
     }
     Q_EMIT started(QPrivateSignal());
 }
 
-void Task::emitError(int errCode, const QString &details)
+void Task::emitError(const GpgME::Error &error, const QString &details)
 {
-    emitResult(makeErrorResult(errCode, details));
+    emitResult(makeErrorResult(error, details));
 }
 
 void Task::emitResult(const std::shared_ptr<const Task::Result> &r)
@@ -204,9 +180,9 @@ void Task::emitResult(const std::shared_ptr<const Task::Result> &r)
     Q_EMIT result(r, QPrivateSignal());
 }
 
-std::shared_ptr<Task::Result> Task::makeErrorResult(int errCode, const QString &details)
+std::shared_ptr<Task::Result> Task::makeErrorResult(const GpgME::Error &error, const QString &details)
 {
-    return std::shared_ptr<Task::Result>(new ErrorResult(errCode, details));
+    return std::shared_ptr<Task::Result>(new ErrorResult(error, details));
 }
 
 class Task::Result::Private
@@ -220,7 +196,7 @@ Task::Result::~Result() {}
 
 bool Task::Result::hasError() const
 {
-    return errorCode() != 0;
+    return error().code() != 0;
 }
 
 static QString image(const char *img)
