@@ -37,7 +37,7 @@ public:
     bool nonRevocableCertificationSelected() const;
 
     void setSelectedUserIDs(const std::vector<GpgME::UserID> &uids);
-    std::vector<unsigned int> selectedUserIDs() const;
+    std::vector<GpgME::UserID> selectedUserIDs() const;
 
     GpgME::Key selectedSecretKey() const;
 
@@ -54,6 +54,9 @@ public:
 public Q_SLOTS:
     void accept() override;
 
+protected:
+    void keyPressEvent(QKeyEvent *) override;
+
 private:
     CertifyWidget *mCertWidget;
 };
