@@ -12,6 +12,8 @@
 #include "certifywidget.h"
 
 #include <utils/accessibility.h>
+#include <utils/keys.h>
+#include "view/infofield.h"
 
 #include "kleopatra_debug.h"
 
@@ -23,6 +25,7 @@
 #include <KSeparator>
 #include <KSharedConfig>
 
+#include <Libkleo/Algorithm>
 #include <Libkleo/DefaultKeyFilter>
 #include <Libkleo/KeySelectionCombo>
 #include <Libkleo/Formatting>
@@ -52,6 +55,11 @@
 
 using namespace Kleo;
 
+static QDebug operator<<(QDebug s, const GpgME::UserID &userID)
+{
+    return s << Formatting::prettyUserID(userID);
+}
+
 namespace {
 
 // Maybe move this in its own file
@@ -60,7 +68,8 @@ class AnimatedExpander: public QWidget
 {
     Q_OBJECT
 public:
-    explicit AnimatedExpander(const QString &title = QString(),
+    explicit AnimatedExpander(const QString &title,
+                              const QString &accessibleTitle = {},
                               QWidget *parent = nullptr);
     void setContentLayout(QLayout *contentLayout);
 
@@ -73,13 +82,24 @@ private:
     int animationDuration{300};
 };
 
-AnimatedExpander::AnimatedExpander(const QString &title, QWidget *parent):
-    QWidget(parent)
+AnimatedExpander::AnimatedExpander(const QString &title, const QString &accessibleTitle, QWidget *parent)
+    : QWidget{parent}
 {
+#ifdef Q_OS_WIN
+    // draw dotted focus frame if button has focus; otherwise, draw invisible frame using background color
+    toggleButton.setStyleSheet(QStringLiteral("QToolButton { border: 1px solid palette(window); }"
+                                              "QToolButton:focus { border: 1px dotted palette(window-text); }"));
+#else
+    // this works with Breeze style because Breeze draws the focus frame when drawing CE_ToolButtonLabel
+    // while the Windows styles (and Qt's common base style) draw the focus frame before drawing CE_ToolButtonLabel
     toggleButton.setStyleSheet(QStringLiteral("QToolButton { border: none; }"));
+#endif
     toggleButton.setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
     toggleButton.setArrowType(Qt::ArrowType::RightArrow);
     toggleButton.setText(title);
+    if (!accessibleTitle.isEmpty()) {
+        toggleButton.setAccessibleName(accessibleTitle);
+    }
     toggleButton.setCheckable(true);
     toggleButton.setChecked(false);
 
@@ -92,6 +112,7 @@ AnimatedExpander::AnimatedExpander(const QString &title, QWidget *parent):
     // start out collapsed
     contentArea.setMaximumHeight(0);
     contentArea.setMinimumHeight(0);
+    contentArea.setVisible(false);
 
     // let the entire widget grow and shrink with its content
     toggleAnimation.addAnimation(new QPropertyAnimation(this, "minimumHeight"));
@@ -106,10 +127,20 @@ AnimatedExpander::AnimatedExpander(const QString &title, QWidget *parent):
     mainLayout.addWidget(&contentArea, row, 0, 1, 3);
     setLayout(&mainLayout);
     QObject::connect(&toggleButton, &QToolButton::clicked, [this](const bool checked) {
+        if (checked) {
+            // make the content visible when expanding starts
+            contentArea.setVisible(true);
+        }
         toggleButton.setArrowType(checked ? Qt::ArrowType::DownArrow : Qt::ArrowType::RightArrow);
         toggleAnimation.setDirection(checked ? QAbstractAnimation::Forward : QAbstractAnimation::Backward);
         toggleAnimation.start();
     });
+    connect(&toggleAnimation, &QAbstractAnimation::finished, [this]() {
+        // hide the content area when it is fully collapsed
+        if (!toggleButton.isChecked()) {
+            contentArea.setVisible(false);
+        }
+    });
 }
 
 void AnimatedExpander::setContentLayout(QLayout *contentLayout)
@@ -162,12 +193,28 @@ private:
     GpgME::Key mExcludedKey;
 };
 
+class UserIDItem : public QStandardItem
+{
+public:
+    explicit UserIDItem(const GpgME::UserID &uid)
+        : mUserId{uid}
+    {}
+
+    GpgME::UserID userId()
+    {
+        return mUserId;
+    }
+
+private:
+    GpgME::UserID mUserId;
+};
+
 class UserIDModel : public QStandardItemModel
 {
     Q_OBJECT
 public:
     enum Role {
-        UserIDIndex = Qt::UserRole
+        UserID = Qt::UserRole
     };
     explicit UserIDModel(QObject *parent = nullptr) : QStandardItemModel(parent) {}
 
@@ -183,14 +230,13 @@ public:
         const std::vector<GpgME::UserID> ids = key.userIDs();
         int i = 0;
         for (const auto &uid: key.userIDs()) {
-            if (uid.isRevoked() || uid.isInvalid()) {
+            if (uid.isRevoked() || uid.isInvalid() || Kleo::isRevokedOrExpired(uid)) {
                 // Skip user IDs that cannot really be certified.
                 i++;
                 continue;
             }
-            auto const item = new QStandardItem;
+            const auto item = new UserIDItem{uid};
             item->setText(Formatting::prettyUserID(uid));
-            item->setData(i, UserIDIndex);
             item->setCheckable(true);
             item->setEditable(false);
             item->setCheckState(Qt::Checked);
@@ -199,38 +245,42 @@ public:
         }
     }
 
-    void setCheckedUserIDs(const std::vector<unsigned int> &uids)
+    void setCheckedUserIDs(const std::vector<GpgME::UserID> &uids)
     {
-        std::vector<unsigned int> sorted = uids;
-        std::sort(sorted.begin(), sorted.end());
         for (int i = 0, end = rowCount(); i != end; ++i) {
-            item(i)->setCheckState(std::binary_search(sorted.begin(), sorted.end(), i) ? Qt::Checked : Qt::Unchecked);
+            const auto uidItem = userIdItem(i);
+            const auto itemUserId = uidItem->userId();
+            const bool userIdIsInList = Kleo::any_of(uids, [itemUserId](const auto &uid) {
+                return Kleo::userIDsAreEqual(itemUserId, uid);
+            });
+            uidItem->setCheckState(userIdIsInList ? Qt::Checked : Qt::Unchecked);
         }
     }
 
-    std::vector<unsigned int> checkedUserIDs() const
+    std::vector<GpgME::UserID> checkedUserIDs() const
     {
-        std::vector<unsigned int> ids;
+        std::vector<GpgME::UserID> userIds;
+        userIds.reserve(rowCount());
         for (int i = 0; i < rowCount(); ++i) {
-            if (item(i)->checkState() == Qt::Checked) {
-                ids.push_back(item(i)->data(UserIDIndex).toUInt());
+            const auto uidItem = userIdItem(i);
+            if (uidItem->checkState() == Qt::Checked) {
+                userIds.push_back(uidItem->userId());
             }
         }
-        qCDebug(KLEOPATRA_LOG) << "Checked uids are: " << ids;
-        return ids;
+        qCDebug(KLEOPATRA_LOG) << "Checked user IDs:" << userIds;
+        return userIds;
+    }
+
+private:
+    UserIDItem *userIdItem(int row) const
+    {
+        return static_cast<UserIDItem *>(item(row));
     }
 
 private:
     GpgME::Key m_key;
 };
 
-static bool uidEqual(const GpgME::UserID &lhs, const GpgME::UserID &rhs)
-{
-    return qstrcmp(lhs.parent().primaryFingerprint(),
-                    rhs.parent().primaryFingerprint()) == 0
-            && qstrcmp(lhs.id(), rhs.id()) == 0;
-}
-
 auto checkBoxSize(const QCheckBox *checkBox)
 {
     QStyleOptionButton opt;
@@ -265,6 +315,32 @@ QString formatDate(const QDate &date, QLocale::FormatType format)
     return QLocale{}.toString(date, dateFormatWithFourDigitYear(format));
 }
 
+class ListView : public QListView
+{
+    Q_OBJECT
+public:
+    using QListView::QListView;
+
+protected:
+    void focusInEvent(QFocusEvent *event) override
+    {
+        QListView::focusInEvent(event);
+        // queue the invokation, so that it happens after the widget itself got focus
+        QMetaObject::invokeMethod(this, &ListView::forceAccessibleFocusEventForCurrentItem, Qt::QueuedConnection);
+    }
+
+private:
+    void forceAccessibleFocusEventForCurrentItem()
+    {
+        // force Qt to send a focus event for the current item to accessibility
+        // tools; otherwise, the user has no idea which item is selected when the
+        // list gets keyboard input focus
+        const auto current = currentIndex();
+        setCurrentIndex({});
+        setCurrentIndex(current);
+    }
+};
+
 }
 
 class CertifyWidget::Private
@@ -275,26 +351,44 @@ public:
     {
         auto mainLay = new QVBoxLayout{q};
 
-        mFprLabel = new QLabel{q};
-        mainLay->addWidget(mFprLabel);
+        {
+            auto label = new QLabel{i18n("Verify the fingerprint, mark the user IDs you want to certify, "
+                "and select the key you want to certify the user IDs with.<br>"
+                "<i>Note: Only the fingerprint clearly identifies the key and its owner.</i>"), q};
+            label->setWordWrap(true);
+            labelHelper.addLabel(label);
+            mainLay->addWidget(label);
+        }
 
-        auto secKeyLay = new QHBoxLayout;
-        secKeyLay->addWidget(new QLabel(i18n("Certify with:")));
+        mainLay->addWidget(new KSeparator{Qt::Horizontal, q});
 
-        mSecKeySelect = new KeySelectionCombo{/* secretOnly= */true, q};
-        mSecKeySelect->setKeyFilter(std::make_shared<SecKeyFilter>());
+        {
+            auto grid = new QGridLayout;
+            grid->setColumnStretch(1, 1);
+            int row = -1;
 
-        secKeyLay->addWidget(mSecKeySelect, 1);
-        mainLay->addLayout(secKeyLay);
+            row++;
+            mFprField = std::make_unique<InfoField>(i18n("Fingerprint:"), q);
+            grid->addWidget(mFprField->label(), row, 0);
+            grid->addLayout(mFprField->layout(), row, 1);
+
+            row++;
+            auto label = new QLabel{i18n("Certify with:"), q};
+            mSecKeySelect = new KeySelectionCombo{/* secretOnly= */true, q};
+            mSecKeySelect->setKeyFilter(std::make_shared<SecKeyFilter>());
+            label->setBuddy(mSecKeySelect);
+            grid->addWidget(label, row, 0);
+            grid->addWidget(mSecKeySelect);
+
+            mainLay->addLayout(grid);
+        }
 
         mMissingOwnerTrustInfo = new KMessageWidget{q};
         mSetOwnerTrustAction = new QAction{q};
         mSetOwnerTrustAction->setText(i18nc("@action:button", "Set Owner Trust"));
-        mSetOwnerTrustAction->setToolTip(QLatin1String("<html>") +
-                                         i18nc("@info:tooltip",
+        mSetOwnerTrustAction->setToolTip(i18nc("@info:tooltip",
                                                "Click to set the trust level of the selected certification key to ultimate trust. "
-                                               "This is what you usually want to do for your own keys.") +
-                                         QLatin1String("</html>"));
+                                               "This is what you usually want to do for your own keys."));
         connect(mSetOwnerTrustAction, &QAction::triggered, q, [this] () { setOwnerTrust(); });
         mMissingOwnerTrustInfo->addAction(mSetOwnerTrustAction);
         mMissingOwnerTrustInfo->setVisible(false);
@@ -303,12 +397,13 @@ public:
 
         mainLay->addWidget(new KSeparator{Qt::Horizontal, q});
 
-        auto listView = new QListView{q};
-        listView->setModel(&mUserIDModel);
-        mainLay->addWidget(listView, 1);
+        userIdListView = new ListView{q};
+        userIdListView->setAccessibleName(i18n("User IDs"));
+        userIdListView->setModel(&mUserIDModel);
+        mainLay->addWidget(userIdListView, 1);
 
         // Setup the advanced area
-        auto expander = new AnimatedExpander{i18n("Advanced"), q};
+        auto expander = new AnimatedExpander{i18n("Advanced"), i18n("Show advanced options"), q};
         mainLay->addWidget(expander);
 
         auto advLay = new QVBoxLayout;
@@ -333,8 +428,10 @@ public:
         {
             auto tagsLay = new QHBoxLayout;
 
+            auto label = new QLabel{i18n("Tags:"), q};
             mTagsLE = new QLineEdit{q};
-            mTagsLE->setPlaceholderText(i18n("Tags"));
+            label->setBuddy(mTagsLE);
+
             auto infoBtn = createInfoButton(i18n("You can use this to add additional info to a certification.") +
                                             QStringLiteral("<br/><br/>") +
                                             i18n("Tags created by anyone with full certification trust "
@@ -342,7 +439,7 @@ public:
                                             q);
             infoBtn->setAccessibleName(i18n("Explain tags"));
 
-            tagsLay->addWidget(new QLabel{i18n("Tags:"), q});
+            tagsLay->addWidget(label);
             tagsLay->addWidget(mTagsLE, 1);
             tagsLay->addWidget(infoBtn);
 
@@ -408,11 +505,14 @@ public:
         {
             auto layout = new QHBoxLayout;
 
+            auto label = new QLabel{i18n("Domain:"), q};
+
             mTrustSignatureDomainLE = new QLineEdit{q};
-            mTrustSignatureDomainLE->setPlaceholderText(i18n("Domain"));
             mTrustSignatureDomainLE->setEnabled(mTrustSignatureCB->isChecked());
+            label->setBuddy(mTrustSignatureDomainLE);
 
             layout->addSpacing(checkBoxSize(mTrustSignatureCB).width());
+            layout->addWidget(label);
             layout->addWidget(mTrustSignatureDomainLE);
 
             advLay->addLayout(layout);
@@ -420,7 +520,9 @@ public:
 
         expander->setContentLayout(advLay);
 
-        connect(&mUserIDModel, &QStandardItemModel::itemChanged, q, &CertifyWidget::changed);
+        connect(&mUserIDModel, &QStandardItemModel::itemChanged, q, [this](QStandardItem *item) {
+            onItemChanged(item);
+        });
 
         connect(mExportCB, &QCheckBox::toggled, [this] (bool on) {
             mPublishCB->setEnabled(on);
@@ -503,9 +605,8 @@ public:
 
     void setTarget(const GpgME::Key &key)
     {
-        mFprLabel->setText(i18n("Fingerprint: <b>%1</b>",
-                            Formatting::prettyID(key.primaryFingerprint())) + QStringLiteral("<br/>") +
-                            i18n("<i>Only the fingerprint clearly identifies the key and its owner.</i>"));
+        mFprField->setValue(QStringLiteral("<b>") + Formatting::prettyID(key.primaryFingerprint()) + QStringLiteral("</b>"),
+                            Formatting::accessibleHexID(key.primaryFingerprint()));
         mUserIDModel.setKey(key);
         mTarget = key;
 
@@ -524,24 +625,10 @@ public:
 
     void selectUserIDs(const std::vector<GpgME::UserID> &uids)
     {
-        const auto all = mTarget.userIDs();
-
-        std::vector<unsigned int> indexes;
-        indexes.reserve(uids.size());
-
-        for (const auto &uid: uids) {
-            const unsigned int idx =
-                std::distance(all.cbegin(), std::find_if(all.cbegin(), all.cend(),
-                            [uid](const GpgME::UserID &other) { return uidEqual(uid, other); }));
-            if (idx < all.size()) {
-                indexes.push_back(idx);
-            }
-        }
-
-        mUserIDModel.setCheckedUserIDs(indexes);
+        mUserIDModel.setCheckedUserIDs(uids);
     }
 
-    std::vector<unsigned int> selectedUserIDs() const
+    std::vector<GpgME::UserID> selectedUserIDs() const
     {
         return mUserIDModel.checkedUserIDs();
     }
@@ -613,7 +700,7 @@ public:
         QGpgME::ChangeOwnerTrustJob *const j = QGpgME::openpgp()->changeOwnerTrustJob();
         connect(j, &QGpgME::ChangeOwnerTrustJob::result, q, [this] (const GpgME::Error &err) {
             if (err) {
-                KMessageBox::sorry(q,
+                KMessageBox::error(q,
                                    i18n("<p>Changing the certification trust of the key <b>%1</b> failed:</p><p>%2</p>",
                                         Formatting::formatForComboBox(secKey()),
                                         QString::fromLocal8Bit(err.asString())),
@@ -630,11 +717,28 @@ public:
         j->start(secKey(), GpgME::Key::Ultimate);
     }
 
+    void onItemChanged(QStandardItem *item)
+    {
+        Q_EMIT q->changed();
+
+#ifndef QT_NO_ACCESSIBILITY
+        if (item) {
+            // assume that the checked state changed
+            QAccessible::State st;
+            st.checked = true;
+            QAccessibleStateChangeEvent e(userIdListView, st);
+            e.setChild(item->index().row());
+            QAccessible::updateAccessibility(&e);
+        }
+#endif
+    }
+
 public:
     CertifyWidget *const q;
-    QLabel *mFprLabel = nullptr;
+    std::unique_ptr<InfoField> mFprField;
     KeySelectionCombo *mSecKeySelect = nullptr;
     KMessageWidget *mMissingOwnerTrustInfo = nullptr;
+    ListView *userIdListView = nullptr;
     QCheckBox *mExportCB = nullptr;
     QCheckBox *mPublishCB = nullptr;
     QLineEdit *mTagsLE = nullptr;
@@ -644,6 +748,8 @@ public:
     KDateComboBox *mExpirationDateEdit = nullptr;
     QAction *mSetOwnerTrustAction = nullptr;
 
+    LabelHelper labelHelper;
+
     UserIDModel mUserIDModel;
     GpgME::Key mTarget;
 };
@@ -671,7 +777,7 @@ void CertifyWidget::selectUserIDs(const std::vector<GpgME::UserID> &uids)
     d->selectUserIDs(uids);
 }
 
-std::vector<unsigned int> CertifyWidget::selectedUserIDs() const
+std::vector<GpgME::UserID> CertifyWidget::selectedUserIDs() const
 {
     return d->selectedUserIDs();
 }
