@@ -114,7 +114,7 @@ private:
     {
         const auto key = ui.treeView->view()->model()->data(index, KeyList::KeyRole).value<GpgME::Key>();
         if (!key.isNull()) {
-            auto cmd = new DetailsCommand(key, nullptr);
+            auto cmd = new DetailsCommand(key);
             cmd->setParentWidget(q);
             cmd->start();
         }
