@@ -8,11 +8,12 @@
     SPDX-License-Identifier: GPL-2.0-or-later
 */
 
-#ifndef __KLEOPATRA_DIALOGS_GROUPDETAILSDIALOG_H__
-#define __KLEOPATRA_DIALOGS_GROUPDETAILSDIALOG_H__
+#pragma once
 
 #include <QDialog>
 
+#include <memory>
+
 namespace Kleo
 {
 class KeyGroup;
@@ -37,4 +38,3 @@ private:
 }
 }
 
-#endif // __KLEOPATRA_DIALOGS_GROUPDETAILSDIALOG_H__
