@@ -30,28 +30,21 @@ class SelfTestDialog : public QDialog
     Q_PROPERTY(bool runAtStartUp READ runAtStartUp WRITE setRunAtStartUp)
 public:
     explicit SelfTestDialog(QWidget *parent = nullptr, Qt::WindowFlags f = {});
-    explicit SelfTestDialog(const std::vector<std::shared_ptr<SelfTest>> &tests, QWidget *parent = nullptr, Qt::WindowFlags f = {});
-    ~SelfTestDialog();
+    ~SelfTestDialog() override;
 
     void setAutomaticMode(bool automatic);
 
-    void addSelfTest(const std::shared_ptr<SelfTest> &test);
-    void addSelfTests(const std::vector<std::shared_ptr<SelfTest>> &tests);
+    void setTests(const std::vector<std::shared_ptr<SelfTest>> &tests);
 
     void setRunAtStartUp(bool run);
     bool runAtStartUp() const;
 
-public Q_SLOTS:
-    void clear();
-
 Q_SIGNALS:
     void updateRequested();
 
 private:
     class Private;
     kdtools::pimpl_ptr<Private> d;
-    Q_PRIVATE_SLOT(d, void slotSelectionChanged())
-    Q_PRIVATE_SLOT(d, void slotDoItClicked())
 };
 
 }
