@@ -12,6 +12,8 @@
 #include "kwatchgnupgconfig.h"
 #include "kwatchgnupg.h"
 
+#include "utils/qt-cxx20-compat.h"
+
 #include <Libkleo/FileNameRequester>
 
 #include <KLocalizedString>
@@ -105,7 +107,7 @@ KWatchGnuPGConfig::KWatchGnuPGConfig(QWidget *parent)
     glay->addWidget(label, row, 0);
     glay->addWidget(mLogLevelCB, row, 1);
 
-    connect(mLogLevelCB, static_cast<void (QComboBox::*)(int)>(&QComboBox::activated), this, &KWatchGnuPGConfig::slotChanged);
+    connect(mLogLevelCB, qOverload<int>(&QComboBox::activated), this, &KWatchGnuPGConfig::slotChanged);
 
     /******************* Log Window group *******************/
     group = new QGroupBox(i18n("Log Window"), top);
@@ -129,7 +131,11 @@ KWatchGnuPGConfig::KWatchGnuPGConfig(QWidget *parent)
     auto button = new QPushButton(i18n("Set &Unlimited"), group);
     glay->addWidget(button, row, 2);
 
-    connect(mLoglenSB, static_cast<void (KPluralHandlingSpinBox::*)(int)>(&KPluralHandlingSpinBox::valueChanged), this, &KWatchGnuPGConfig::slotChanged);
+#if QT_DEPRECATED_SINCE(5, 14)
+    connect(mLoglenSB, qOverload<int>(&QSpinBox::valueChanged), this, &KWatchGnuPGConfig::slotChanged);
+#else
+    connect(mLoglenSB, &QSpinBox::valueChanged, this, &KWatchGnuPGConfig::slotChanged);
+#endif
     connect(button, &QPushButton::clicked, this, &KWatchGnuPGConfig::slotSetHistorySizeUnlimited);
 
     ++row;
