@@ -11,7 +11,9 @@
 
 #include "aboutdata.h"
 #include "kwatchgnupgmainwin.h"
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
 #include <Kdelibs4ConfigMigrator>
+#endif
 #include "utils/kuniqueservice.h"
 
 #include <KLocalizedString>
@@ -22,14 +24,18 @@
 
 int main(int argc, char **argv)
 {
-    QCoreApplication::setAttribute(Qt::AA_UseHighDpiPixmaps, true);
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
     QCoreApplication::setAttribute(Qt::AA_EnableHighDpiScaling, true);
+    QCoreApplication::setAttribute(Qt::AA_UseHighDpiPixmaps, true);
+#endif
     QApplication app(argc, argv);
     KCrash::initialize();
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
     Kdelibs4ConfigMigrator migrate(QStringLiteral("kwatchgnupg"));
     migrate.setConfigFiles(QStringList() << QStringLiteral("kwatchgnupgrc"));
     migrate.setUiFiles(QStringList() << QStringLiteral("kwatchgnupgui.rc"));
     migrate.migrate();
+#endif
 
     KLocalizedString::setApplicationDomain("kwatchgnupg");
     AboutData aboutData;
@@ -42,7 +48,7 @@ int main(int argc, char **argv)
 
     KUniqueService service;
 
-    KWatchGnuPGMainWindow *mMainWin = new KWatchGnuPGMainWindow();
+    auto mMainWin = new KWatchGnuPGMainWindow();
     mMainWin->show();
     return app.exec();
 }
