@@ -59,6 +59,7 @@
 #include <KConfigGroup>
 #include <KConfigDialog>
 #include <KColorScheme>
+#include <kwidgetsaddons_version.h>
 
 #include <QAbstractItemView>
 #include <QCloseEvent>
@@ -72,7 +73,9 @@
 #include <QStackedWidget>
 #include <QStatusBar>
 #include <QLabel>
+#include <QPixmap>
 
+#include <Libkleo/Compliance>
 #include <Libkleo/Formatting>
 #include <Libkleo/GnuPG>
 #include <Libkleo/KeyListModel>
@@ -196,14 +199,19 @@ public:
     void closeAndQuit()
     {
         const QString app = KAboutData::applicationData().displayName();
+#if KWIDGETSADDONS_VERSION >= QT_VERSION_CHECK(5, 100, 0)
+        const int rc = KMessageBox::questionTwoActionsCancel(q,
+#else
         const int rc = KMessageBox::questionYesNoCancel(q,
-                       xi18n("<application>%1</application> may be used by other applications as a service.<nl/>"
-                             "You may instead want to close this window without exiting <application>%1</application>.", app),
-                       i18nc("@title:window", "Really Quit?"),
-                       KStandardGuiItem_close(),
-                       KStandardGuiItem_quit(),
-                       KStandardGuiItem::cancel(),
-                       QLatin1String("really-quit-") + app.toLower());
+#endif
+                                                             xi18n("<application>%1</application> may be used by other applications as a service.<nl/>"
+                                                                   "You may instead want to close this window without exiting <application>%1</application>.",
+                                                                   app),
+                                                             i18nc("@title:window", "Really Quit?"),
+                                                             KStandardGuiItem_close(),
+                                                             KStandardGuiItem_quit(),
+                                                             KStandardGuiItem::cancel(),
+                                                             QLatin1String("really-quit-") + app.toLower());
         if (rc == KMessageBox::Cancel) {
             return;
         }
@@ -212,7 +220,11 @@ public:
             return;
         }
         // WARNING: 'this' might be deleted at this point!
+#if KWIDGETSADDONS_VERSION >= QT_VERSION_CHECK(5, 100, 0)
+        if (rc == KMessageBox::ButtonCode::SecondaryAction) {
+#else
         if (rc == KMessageBox::No) {
+#endif
             qApp->quit();
         }
     }
@@ -235,15 +247,15 @@ public:
 
     void updateStatusBar()
     {
-        if (Kleo::gnupgUsesDeVsCompliance()) {
+        if (DeVSCompliance::isActive()) {
             auto statusBar = std::make_unique<QStatusBar>();
-            auto statusLbl = std::make_unique<QLabel>(Formatting::deVsString(Kleo::gnupgIsDeVsCompliant()));
+            auto statusLbl = std::make_unique<QLabel>(DeVSCompliance::name());
             if (!SystemInfo::isHighContrastModeActive()) {
                 const auto color = KColorScheme(QPalette::Active, KColorScheme::View).foreground(
-                    Kleo::gnupgIsDeVsCompliant() ? KColorScheme::NormalText: KColorScheme::NegativeText
+                    DeVSCompliance::isCompliant() ? KColorScheme::NormalText: KColorScheme::NegativeText
                 ).color();
                 const auto background = KColorScheme(QPalette::Active, KColorScheme::View).background(
-                    Kleo::gnupgIsDeVsCompliant() ? KColorScheme::PositiveBackground : KColorScheme::NegativeBackground
+                    DeVSCompliance::isCompliant() ? KColorScheme::PositiveBackground : KColorScheme::NegativeBackground
                 ).color();
                 statusLbl->setStyleSheet(QStringLiteral("QLabel { color: %1; background-color: %2; }").
                                         arg(color.name()).arg(background.name()));
@@ -440,7 +452,8 @@ MainWindow::Private::Private(MainWindow *qq)
     q->createGUI(QStringLiteral("kleopatra.rc"));
 
     // make toolbar buttons accessible by keyboard
-    if (auto toolbar = q->findChild<KToolBar*>()) {
+    auto toolbar = q->findChild<KToolBar*>();
+    if (toolbar) {
         auto toolbarButtons = toolbar->findChildren<QToolButton*>();
         for (auto b : toolbarButtons) {
             b->setFocusPolicy(Qt::TabFocus);
@@ -454,6 +467,28 @@ MainWindow::Private::Private(MainWindow *qq)
         });
     }
 
+    const auto title = Kleo::brandingWindowTitle();
+    if (!title.isEmpty()) {
+        QApplication::setApplicationDisplayName(title);
+    }
+
+    const auto icon = Kleo::brandingIcon();
+    if (!icon.isEmpty()) {
+        const auto dir = QDir(Kleo::gpg4winInstallPath() + QStringLiteral("/../share/kleopatra/pics"));
+        qCDebug(KLEOPATRA_LOG) << "Loading branding icon:" << dir.absoluteFilePath(icon);
+        QPixmap brandingIcon(dir.absoluteFilePath(icon));
+        if (!brandingIcon.isNull()) {
+            auto *w = new QWidget;
+            auto *hl = new QHBoxLayout;
+            auto *lbl = new QLabel;
+            w->setLayout(hl);
+            hl->addWidget(lbl);
+            lbl->setPixmap(brandingIcon);
+            toolbar->addSeparator();
+            toolbar->addWidget(w);
+        }
+    }
+
     q->setAcceptDrops(true);
 
     // set default window size
@@ -592,6 +627,12 @@ void MainWindow::Private::setupActions()
             QStringLiteral("../share/doc/gnupg-vsd"));
     coll->addAction(QStringLiteral("help_doc_vsa10584"), vsa10584);
 
+    const auto smartcard = new DocAction(QIcon::fromTheme(QStringLiteral("dvipdf")), i18n("Smartcard Management"),
+            i18nc("Only available in German and English. Leave to English for other languages.",
+                "smartcard_howto_de.pdf"),
+            QStringLiteral("../share/doc/gnupg-vsd"));
+    coll->addAction(QStringLiteral("help_doc_smartcard"), smartcard);
+
     const auto man_gpg = new DocAction(QIcon::fromTheme(QStringLiteral("help-contextual")), i18n("GnuPG Manual"),
             QStringLiteral("gnupg.pdf"), QStringLiteral("../share/doc/gnupg"));
     coll->addAction(QStringLiteral("help_doc_gnupg"), man_gpg);
