@@ -15,6 +15,8 @@
 
 #include <gpgme++/global.h>
 
+class QDir;
+
 namespace Kleo
 {
 
@@ -34,9 +36,7 @@ public:
     GpgME::Protocol protocol() const;
 
     enum Pages {
-        ChooseProtocolPageId,
         EnterDetailsPageId,
-        OverviewPageId,
         KeyCreationPageId,
         ResultPageId,
 
@@ -49,6 +49,7 @@ protected:
 private:
     void resetProtocol();
     void restartAtEnterDetailsPage();
+    QDir tmpDir() const;
 
 private:
     class Private;
