@@ -9,6 +9,7 @@
 
 #include "pivcard.h"
 
+#include "algorithminfo.h"
 #include "keypairinfo.h"
 
 #include <KLocalizedString>
@@ -91,7 +92,7 @@ QString PIVCard::keyDisplayName(const std::string &keyRef)
 }
 
 // static
-std::vector< std::pair<std::string, QString> > PIVCard::supportedAlgorithms(const std::string &keyRef)
+std::vector<AlgorithmInfo> PIVCard::supportedAlgorithms(const std::string &keyRef)
 {
     if (keyRef == PIVCard::keyManagementKeyRef()) {
         return {
@@ -114,45 +115,22 @@ std::vector< std::pair<std::string, QString> > PIVCard::supportedAlgorithms(cons
     };
 }
 
-void PIVCard::setCardInfo(const std::vector< std::pair<std::string, std::string> > &infos)
-{
-    qCDebug(KLEOPATRA_LOG) << "Card" << serialNumber().c_str() << "info:";
-    for (const auto &pair: infos) {
-        qCDebug(KLEOPATRA_LOG) << pair.first.c_str() << ":" << pair.second.c_str();
-        if (parseCardInfo(pair.first, pair.second)) {
-            continue;
-        }
-        mMetaInfo.insert(pair.first, pair.second);
-    }
-}
-
 std::string PIVCard::keyAlgorithm(const std::string &keyRef) const
 {
-    return mMetaInfo.value("KLEO-KEYALGO-" + keyRef);
+    return cardInfo("KLEO-KEYALGO-" + keyRef);
 }
 
 void PIVCard::setKeyAlgorithm(const std::string &keyRef, const std::string &algorithm)
 {
-    mMetaInfo.insert("KLEO-KEYALGO-" + keyRef, algorithm);
+    addCardInfo("KLEO-KEYALGO-" + keyRef, algorithm);
 }
 
 std::string PIVCard::certificateData(const std::string &keyRef) const
 {
-    return mMetaInfo.value("KLEO-CERTIFICATE-" + keyRef);
+    return cardInfo("KLEO-CERTIFICATE-" + keyRef);
 }
 
 void PIVCard::setCertificateData(const std::string &keyRef, const std::string &data)
 {
-    mMetaInfo.insert("KLEO-CERTIFICATE-" + keyRef, data);
-}
-
-bool PIVCard::operator == (const Card& rhs) const
-{
-    const PIVCard *other = dynamic_cast<const PIVCard *>(&rhs);
-    if (!other) {
-        return false;
-    }
-
-    return Card::operator ==(rhs)
-        && mMetaInfo == other->mMetaInfo;
+    addCardInfo("KLEO-CERTIFICATE-" + keyRef, data);
 }
