@@ -18,6 +18,8 @@
 #include <Libkleo/Assuan>
 #include <Libkleo/GnuPG>
 #include <Libkleo/FileSystemWatcher>
+#include <Libkleo/KeyCache>
+#include <Libkleo/Stl_Util>
 
 #include <QGpgME/Debug>
 
@@ -71,6 +73,11 @@ static_assert(sizeof prettyFlags / sizeof * prettyFlags == Card::NumStates, "");
 
 Q_DECLARE_METATYPE(GpgME::Error)
 
+static QDebug operator<<(QDebug s, const std::string &string)
+{
+    return s << QString::fromStdString(string);
+}
+
 namespace
 {
 static bool gpgHasMultiCardMultiAppSupport()
@@ -78,11 +85,6 @@ static bool gpgHasMultiCardMultiAppSupport()
     return !(engineInfo(GpgME::GpgEngine).engineVersion() < "2.3.0");
 }
 
-static QDebug operator<<(QDebug s, const std::string &string)
-{
-    return s << QString::fromStdString(string);
-}
-
 static QDebug operator<<(QDebug s, const std::vector< std::pair<std::string, std::string> > &v)
 {
     using pair = std::pair<std::string, std::string>;
@@ -273,6 +275,84 @@ static const char * get_openpgp_card_manufacturer_from_serial_number(const std::
     }
 }
 
+static std::vector<std::string> get_openpgp_card_supported_algorithms_announced_by_card(std::shared_ptr<Context> &gpgAgent)
+{
+    static constexpr std::string_view cardSlotPrefix = "OPENPGP.1 ";
+    static const std::map<std::string_view, std::string_view> algoMapping = {
+        {"cv25519", "curve25519"},
+        {"cv448", "curve448"},
+        {"ed25519", "curve25519"},
+        {"ed448", "curve448"},
+        {"x448", "curve448"},
+    };
+
+    Error err;
+    const auto lines = Assuan::sendStatusLinesCommand(gpgAgent, "SCD GETATTR KEY-ATTR-INFO", err);
+    if (err) {
+        return {};
+    }
+
+    std::vector<std::string> algos;
+    kdtools::transform_if(lines.cbegin(),
+                          lines.cend(),
+                          std::back_inserter(algos),
+                          [](const auto &line) {
+                              auto algo = line.second.substr(cardSlotPrefix.size());
+                              // map a few algorithms to the standard names used by us
+                              const auto mapping = algoMapping.find(algo);
+                              if (mapping != algoMapping.end()) {
+                                  algo = mapping->second;
+                              }
+                              return algo;
+                          },
+                          [](const auto &line) {
+                              // only consider KEY-ATTR-INFO status lines for the first card slot;
+                              // for now, we assume that all card slots support the same algorithms
+                              return line.first == "KEY-ATTR-INFO" && line.second.starts_with(cardSlotPrefix);
+                          });
+    // remove duplicate algorithms
+    std::sort(algos.begin(), algos.end());
+    algos.erase(std::unique(algos.begin(), algos.end()), algos.end());
+    qCDebug(KLEOPATRA_LOG) << __func__ << "returns" << algos;
+    return algos;
+}
+
+static std::vector<std::string> get_openpgp_card_supported_algorithms(Card *card, std::shared_ptr<Context> &gpgAgent)
+{
+    // first ask the smart card for the supported algorithms
+    const std::vector<std::string> announcedAlgos = get_openpgp_card_supported_algorithms_announced_by_card(gpgAgent);
+    if (!announcedAlgos.empty()) {
+        return announcedAlgos;
+    }
+
+    // otherwise, fall back to hard-coded lists
+    if ((card->cardType() == "yubikey") && (card->cardVersion() >= 0x050203)) {
+        return {
+            "rsa2048",
+            "rsa3072",
+            "rsa4096",
+            "brainpoolP256r1",
+            "brainpoolP384r1",
+            "brainpoolP512r1",
+            "curve25519",
+        };
+    } else if ((card->cardType() == "zeitcontrol") && (card->appVersion() >= 0x0304)) {
+        return {
+            "rsa2048",
+            "rsa3072",
+            "rsa4096",
+            "brainpoolP256r1",
+            "brainpoolP384r1",
+            "brainpoolP512r1",
+        };
+    }
+    return {
+       "rsa2048",
+       "rsa3072",
+       "rsa4096"
+    };
+}
+
 static bool isOpenPGPCardSerialNumber(const std::string &serialNumber)
 {
     return serialNumber.size() == 32 && serialNumber.substr(0, 12) == "D27600012401";
@@ -341,6 +421,8 @@ static void handle_openpgp_card(std::shared_ptr<Card> &ci, std::shared_ptr<Conte
 
     learnCardKeyStubs(pgpCard, gpg_agent);
 
+    pgpCard->setSupportedAlgorithms(get_openpgp_card_supported_algorithms(pgpCard, gpg_agent));
+
     ci.reset(pgpCard);
 }
 
@@ -628,7 +710,7 @@ struct Transaction {
     CardApp cardApp;
     QByteArray command;
     QPointer<QObject> receiver;
-    const char *slot;
+    ReaderStatus::TransactionFunc slot;
     AssuanTransaction* assuanTransaction;
 };
 
@@ -708,8 +790,8 @@ private Q_SLOTS:
         KDAB_SYNCHRONIZED(m_mutex)
         ft.splice(ft.begin(), m_finishedTransactions);
         for (const Transaction &t : std::as_const(ft))
-            if (t.receiver && t.slot && *t.slot) {
-                QMetaObject::invokeMethod(t.receiver, t.slot, Qt::DirectConnection, Q_ARG(GpgME::Error, err));
+            if (t.receiver && t.slot) {
+                QMetaObject::invokeMethod(t.receiver, [&t, &err]() { t.slot(err); }, Qt::DirectConnection);
             }
     }
 
@@ -725,6 +807,11 @@ private:
             std::list<Transaction> item;
             std::vector<std::shared_ptr<Card> > oldCards;
 
+            while (!KeyCache::instance()->initialized()) {
+                qCDebug(KLEOPATRA_LOG) << "Waiting for Keycache to be initialized.";
+                sleep(1);
+            }
+
             Error err;
             std::unique_ptr<Context> c = Context::createForEngine(AssuanEngine, &err);
             if (err.code() == GPG_ERR_NOT_SUPPORTED) {
@@ -967,14 +1054,17 @@ bool ReaderStatus::anyCardCanLearnKeys() const
     return d->anyCardCanLearnKeysImpl();
 }
 
-void ReaderStatus::startSimpleTransaction(const std::shared_ptr<Card> &card, const QByteArray &command, QObject *receiver, const char *slot)
+void ReaderStatus::startSimpleTransaction(const std::shared_ptr<Card> &card, const QByteArray &command, QObject *receiver, const TransactionFunc &slot)
 {
     const CardApp cardApp = { card->serialNumber(), card->appName() };
     const Transaction t = { cardApp, command, receiver, slot, nullptr };
     d->addTransaction(t);
 }
 
-void ReaderStatus::startTransaction(const std::shared_ptr<Card> &card, const QByteArray &command, QObject *receiver, const char *slot,
+void ReaderStatus::startTransaction(const std::shared_ptr<Card> &card,
+                                    const QByteArray &command,
+                                    QObject *receiver,
+                                    const TransactionFunc &slot,
                                     std::unique_ptr<AssuanTransaction> transaction)
 {
     const CardApp cardApp = { card->serialNumber(), card->appName() };
