@@ -1,5 +1,5 @@
 /*
-  SPDX-FileCopyrightText: 2014-2020 Laurent Montel <montel@kde.org>
+  SPDX-FileCopyrightText: 2014-2021 Laurent Montel <montel@kde.org>
 
   SPDX-License-Identifier: GPL-2.0-only
 */
@@ -9,11 +9,16 @@
 #include "kdtoolsglobal.h"
 #include "mainwindow.h"
 
+#include <settings.h>
+
 #include <commands/importcertificatefromclipboardcommand.h>
 #include <commands/encryptclipboardcommand.h>
 #include <commands/signclipboardcommand.h>
 #include <commands/decryptverifyclipboardcommand.h>
 
+#include <Libkleo/Algorithm>
+#include <Libkleo/KeyCache>
+
 #include <KLocalizedString>
 #include <KActionMenu>
 
@@ -22,18 +27,22 @@
 #include <QClipboard>
 #include <QSignalBlocker>
 
+#include <gpgme++/key.h>
+
 using namespace Kleo;
 
 using namespace Kleo::Commands;
 
 ClipboardMenu::ClipboardMenu(QObject *parent)
-    : QObject(parent),
-      mWindow(nullptr)
+    : QObject{parent}
 {
     mClipboardMenu = new KActionMenu(i18n("Clipboard"), this);
     mImportClipboardAction = new QAction(i18n("Certificate Import"), this);
     mEncryptClipboardAction = new QAction(i18n("Encrypt..."), this);
-    mSmimeSignClipboardAction = new QAction(i18n("S/MIME-Sign..."), this);
+    const Kleo::Settings settings{};
+    if (settings.cmsEnabled() && settings.cmsSigningAllowed()) {
+        mSmimeSignClipboardAction = new QAction(i18n("S/MIME-Sign..."), this);
+    }
     mOpenPGPSignClipboardAction = new QAction(i18n("OpenPGP-Sign..."), this);
     mDecryptVerifyClipboardAction = new QAction(i18n("Decrypt/Verify..."), this);
 
@@ -46,22 +55,24 @@ ClipboardMenu::ClipboardMenu(QObject *parent)
 
     connect(mImportClipboardAction, &QAction::triggered, this, &ClipboardMenu::slotImportClipboard);
     connect(mEncryptClipboardAction, &QAction::triggered, this, &ClipboardMenu::slotEncryptClipboard);
-    connect(mSmimeSignClipboardAction, &QAction::triggered, this, &ClipboardMenu::slotSMIMESignClipboard);
+    if (mSmimeSignClipboardAction) {
+        connect(mSmimeSignClipboardAction, &QAction::triggered, this, &ClipboardMenu::slotSMIMESignClipboard);
+    }
     connect(mOpenPGPSignClipboardAction, &QAction::triggered, this, &ClipboardMenu::slotOpenPGPSignClipboard);
     connect(mDecryptVerifyClipboardAction, &QAction::triggered, this, &ClipboardMenu::slotDecryptVerifyClipboard);
     mClipboardMenu->addAction(mImportClipboardAction);
     mClipboardMenu->addAction(mEncryptClipboardAction);
-    mClipboardMenu->addAction(mSmimeSignClipboardAction);
+    if (mSmimeSignClipboardAction) {
+        mClipboardMenu->addAction(mSmimeSignClipboardAction);
+    }
     mClipboardMenu->addAction(mOpenPGPSignClipboardAction);
     mClipboardMenu->addAction(mDecryptVerifyClipboardAction);
     connect(QApplication::clipboard(), &QClipboard::changed, this, &ClipboardMenu::slotEnableDisableActions);
+    connect(KeyCache::instance().get(), &KeyCache::keyListingDone, this, &ClipboardMenu::slotEnableDisableActions);
     slotEnableDisableActions();
 }
 
-ClipboardMenu::~ClipboardMenu()
-{
-
-}
+ClipboardMenu::~ClipboardMenu() = default;
 
 void ClipboardMenu::setMainWindow(MainWindow *window)
 {
@@ -105,12 +116,29 @@ void ClipboardMenu::slotDecryptVerifyClipboard()
     startCommand(new DecryptVerifyClipboardCommand(nullptr));
 }
 
+namespace
+{
+
+bool hasSigningKeys(GpgME::Protocol protocol) {
+    if (!KeyCache::instance()->initialized()) {
+        return false;
+    }
+    return Kleo::any_of(KeyCache::instance()->keys(),
+                        [protocol](const auto &k) {
+                            return k.hasSecret() && k.canReallySign() && (k.protocol() == protocol);
+                        });
+}
+
+}
+
 void ClipboardMenu::slotEnableDisableActions()
 {
     const QSignalBlocker blocker(QApplication::clipboard());
     mImportClipboardAction->setEnabled(ImportCertificateFromClipboardCommand::canImportCurrentClipboard());
     mEncryptClipboardAction->setEnabled(EncryptClipboardCommand::canEncryptCurrentClipboard());
-    mOpenPGPSignClipboardAction->setEnabled(SignClipboardCommand::canSignCurrentClipboard());
-    mSmimeSignClipboardAction->setEnabled(SignClipboardCommand::canSignCurrentClipboard());
+    mOpenPGPSignClipboardAction->setEnabled(SignClipboardCommand::canSignCurrentClipboard() && hasSigningKeys(GpgME::OpenPGP));
+    if (mSmimeSignClipboardAction) {
+        mSmimeSignClipboardAction->setEnabled(SignClipboardCommand::canSignCurrentClipboard() && hasSigningKeys(GpgME::CMS));
+    }
     mDecryptVerifyClipboardAction->setEnabled(DecryptVerifyClipboardCommand::canDecryptVerifyCurrentClipboard());
 }
