@@ -15,11 +15,19 @@
 
 #include "smartcard/netkeycard.h"
 #include "smartcard/readerstatus.h"
+
+#include "commands/changepincommand.h"
+#include "commands/createopenpgpkeyfromcardkeyscommand.h"
+#include "commands/createcsrforcardkeycommand.h"
 #include "commands/learncardkeyscommand.h"
 #include "commands/detailscommand.h"
 
 #include <Libkleo/KeyListModel>
 
+#include <KConfigGroup>
+#include <KSharedConfig>
+
+#include <QInputDialog>
 #include <QLabel>
 #include <QVBoxLayout>
 #include <QHBoxLayout>
@@ -30,6 +38,9 @@
 #include <KLocalizedString>
 #include <KMessageBox>
 
+#include <gpgme++/context.h>
+#include <gpgme++/engineinfo.h>
+
 using namespace Kleo;
 using namespace Kleo::SmartCard;
 using namespace Kleo::Commands;
@@ -40,7 +51,7 @@ NetKeyWidget::NetKeyWidget(QWidget *parent) :
     mVersionLabel(new QLabel(this)),
     mLearnKeysLabel(new QLabel(this)),
     mErrorLabel(new QLabel(this)),
-    mNullPinWidget(new NullPinWidget()),
+    mNullPinWidget(new NullPinWidget(this)),
     mLearnKeysBtn(new QPushButton(this)),
     mChangeNKSPINBtn(new QPushButton(this)),
     mChangeSigGPINBtn(new QPushButton(this)),
@@ -117,7 +128,7 @@ NetKeyWidget::NetKeyWidget(QWidget *parent) :
             qCDebug(KLEOPATRA_LOG) << "Unhandled Model: " << mTreeView->view()->model()->metaObject()->className();
             return;
         }
-        auto cmd = new DetailsCommand(klm->key(idx), nullptr);
+        auto cmd = new DetailsCommand(klm->key(idx));
         cmd->setParentWidget(this);
         cmd->start();
     });
@@ -130,36 +141,74 @@ NetKeyWidget::NetKeyWidget(QWidget *parent) :
     vLay->addWidget(line2);
     vLay->addWidget(new QLabel(QStringLiteral("<b>%1</b>").arg(i18n("Actions:"))), 0, Qt::AlignLeft);
 
+    auto actionLayout = new QHBoxLayout();
+
+    if (CreateOpenPGPKeyFromCardKeysCommand::isSupported()) {
+        mKeyForCardKeysButton = new QPushButton(this);
+        mKeyForCardKeysButton->setText(i18nc("@action:button", "Create OpenPGP Key"));
+        mKeyForCardKeysButton->setToolTip(i18nc("@info:tooltip", "Create an OpenPGP key for the keys stored on the card."));
+        actionLayout->addWidget(mKeyForCardKeysButton);
+        connect(mKeyForCardKeysButton, &QPushButton::clicked, this, &NetKeyWidget::createKeyFromCardKeys);
+    }
+
+    if (!(engineInfo(GpgME::GpgSMEngine).engineVersion() < "2.2.26")) { // see https://dev.gnupg.org/T5184
+        mCreateCSRButton = new QPushButton(this);
+        mCreateCSRButton->setText(i18nc("@action:button", "Create CSR"));
+        mCreateCSRButton->setToolTip(i18nc("@info:tooltip", "Create a certificate signing request for a key stored on the card."));
+        mCreateCSRButton->setEnabled(false);
+        actionLayout->addWidget(mCreateCSRButton);
+        connect(mCreateCSRButton, &QPushButton::clicked, this, [this] () { createCSR(); });
+    }
+
     mChangeNKSPINBtn->setText(i18nc("NKS is an identifier for a type of keys on a NetKey card", "Change NKS PIN"));
     mChangeSigGPINBtn->setText(i18nc("SigG is an identifier for a type of keys on a NetKey card", "Change SigG PIN"));
 
-    connect(mChangeNKSPINBtn, &QPushButton::clicked, this, [this] () {
-            mChangeNKSPINBtn->setEnabled(false);
-            doChangePin(false);
-        });
-    connect(mChangeSigGPINBtn, &QPushButton::clicked, this, [this] () {
-            mChangeSigGPINBtn->setEnabled(false);
-            doChangePin(true);
-        });
+    connect(mChangeNKSPINBtn, &QPushButton::clicked, this, [this] () { doChangePin(NetKeyCard::nksPinKeyRef()); });
+    connect(mChangeSigGPINBtn, &QPushButton::clicked, this, [this] () { doChangePin(NetKeyCard::sigGPinKeyRef()); });
 
-    auto hLay3 = new QHBoxLayout();
-    hLay3->addWidget(mChangeNKSPINBtn);
-    hLay3->addWidget(mChangeSigGPINBtn);
-    hLay3->addStretch(1);
+    actionLayout->addWidget(mChangeNKSPINBtn);
+    actionLayout->addWidget(mChangeSigGPINBtn);
+    actionLayout->addStretch(1);
 
-    vLay->addLayout(hLay3);
+    vLay->addLayout(actionLayout);
     vLay->addStretch(1);
+
+    const KConfigGroup configGroup(KSharedConfig::openConfig(), "NetKeyCardView");
+    mTreeView->restoreLayout(configGroup);
 }
 
 NetKeyWidget::~NetKeyWidget()
 {
+    KConfigGroup configGroup(KSharedConfig::openConfig(), "NetKeyCardView");
+    mTreeView->saveLayout(configGroup);
+}
+
+namespace
+{
+std::vector<KeyPairInfo> getKeysSuitableForCSRCreation(const NetKeyCard *netKeyCard)
+{
+    std::vector<KeyPairInfo> keys;
+
+    for (const auto &key : netKeyCard->keyInfos()) {
+        if (key.keyRef.substr(0, 9) == "NKS-SIGG.") {
+            // SigG certificates for qualified signatures are issued with the physical cards;
+            // it's not possible to request a certificate for them
+            continue;
+        }
+        if (key.canSign() && (key.keyRef.substr(0, 9) == "NKS-NKS3.") && !netKeyCard->hasNKSNullPin()) {
+            keys.push_back(key);
+        }
+    }
+
+    return keys;
+}
 }
 
 void NetKeyWidget::setCard(const NetKeyCard* card)
 {
     mSerialNumber = card->serialNumber();
     mVersionLabel->setText(i18nc("1 is a Version number", "NetKey v%1 Card", card->appVersion()));
-    mSerialNumberLabel->setText(QString::fromStdString(mSerialNumber));
+    mSerialNumberLabel->setText(card->displaySerialNumber());
 
     mNullPinWidget->setSerialNumber(mSerialNumber);
     /* According to users of NetKey Cards it is fairly uncommon
@@ -193,45 +242,88 @@ void NetKeyWidget::setCard(const NetKeyCard* card)
 
     const auto keys = card->keys();
     mTreeView->setKeys(keys);
-}
 
-void NetKeyWidget::handleResult(const GpgME::Error &err, QPushButton *btn)
-{
-    btn->setEnabled(true);
-    if (err.isCanceled()) {
-        return;
+    if (mKeyForCardKeysButton) {
+        mKeyForCardKeysButton->setEnabled(!card->hasNKSNullPin() && card->hasSigningKey() && card->hasEncryptionKey());
     }
-    if (err) {
-        KMessageBox::error(this, i18nc("@info",
-                           "Failed to set PIN: %1", QString::fromLatin1(err.asString())),
-                           i18nc("@title", "Error"));
-        return;
+    if (mCreateCSRButton) {
+        mCreateCSRButton->setEnabled(!getKeysSuitableForCSRCreation(card).empty());
     }
 }
 
-void NetKeyWidget::setSigGPinSettingResult(const GpgME::Error &err)
+void NetKeyWidget::doChangePin(const std::string &keyRef)
 {
-    handleResult(err, mChangeSigGPINBtn);
-}
-
-void NetKeyWidget::setNksPinSettingResult(const GpgME::Error &err)
-{
-    handleResult(err, mChangeNKSPINBtn);
-}
-
-void NetKeyWidget::doChangePin(bool sigG)
-{
-    const auto nksCard = ReaderStatus::instance()->getCard<NetKeyCard>(mSerialNumber);
-    if (!nksCard) {
-        KMessageBox::error(this, i18n("Failed to find the NetKey card with the serial number: %1", QString::fromStdString(mSerialNumber)));
+    const auto netKeyCard = ReaderStatus::instance()->getCard<NetKeyCard>(mSerialNumber);
+    if (!netKeyCard) {
+        KMessageBox::error(this,
+                           i18n("Failed to find the smartcard with the serial number: %1", QString::fromStdString(mSerialNumber)));
         return;
     }
 
-    if (sigG) {
-        ReaderStatus::mutableInstance()->startSimpleTransaction(
-            nksCard, "SCD PASSWD PW1.CH.SIG", this, "setSigGPinSettingResult");
-    } else {
-        ReaderStatus::mutableInstance()->startSimpleTransaction(
-            nksCard, "SCD PASSWD PW1.CH", this, "setNksPinSettingResult");
+    auto cmd = new ChangePinCommand(mSerialNumber, NetKeyCard::AppName, this);
+    this->setEnabled(false);
+    connect(cmd, &ChangePinCommand::finished,
+            this, [this]() {
+                this->setEnabled(true);
+            });
+    cmd->setKeyRef(keyRef);
+    if ((keyRef == NetKeyCard::nksPinKeyRef() && netKeyCard->hasNKSNullPin())
+        || (keyRef == NetKeyCard::sigGPinKeyRef() && netKeyCard->hasSigGNullPin())) {
+        cmd->setMode(ChangePinCommand::NullPinMode);
     }
+    cmd->start();
+}
+
+void NetKeyWidget::createKeyFromCardKeys()
+{
+    auto cmd = new CreateOpenPGPKeyFromCardKeysCommand(mSerialNumber, NetKeyCard::AppName, this);
+    this->setEnabled(false);
+    connect(cmd, &CreateOpenPGPKeyFromCardKeysCommand::finished,
+            this, [this]() {
+                this->setEnabled(true);
+            });
+    cmd->start();
+}
+
+namespace
+{
+std::string getKeyRef(const std::vector<KeyPairInfo> &keys, QWidget *parent)
+{
+    QStringList options;
+    for (const auto &key : keys) {
+        options << QStringLiteral("%1 - %2").arg(QString::fromStdString(key.keyRef), QString::fromStdString(key.grip));
+    }
+
+    bool ok;
+    const QString choice = QInputDialog::getItem(parent, i18n("Select Key"),
+        i18n("Please select the key you want to create a certificate signing request for:"), options, /* current= */ 0, /* editable= */ false, &ok);
+    return ok ? keys[options.indexOf(choice)].keyRef : std::string();
+}
+}
+
+void NetKeyWidget::createCSR()
+{
+    const auto netKeyCard = ReaderStatus::instance()->getCard<NetKeyCard>(mSerialNumber);
+    if (!netKeyCard) {
+        KMessageBox::error(this,
+                           i18n("Failed to find the smartcard with the serial number: %1", QString::fromStdString(mSerialNumber)));
+        return;
+    }
+    const auto suitableKeys = getKeysSuitableForCSRCreation(netKeyCard.get());
+    if (suitableKeys.empty()) {
+        KMessageBox::error(this,
+                           i18n("Sorry! No keys suitable for creating a certificate signing request found on the smartcard."));
+        return;
+    }
+    const auto keyRef = getKeyRef(suitableKeys, this);
+    if (keyRef.empty()) {
+        return;
+    }
+    auto cmd = new CreateCSRForCardKeyCommand(keyRef, mSerialNumber, NetKeyCard::AppName, this);
+    this->setEnabled(false);
+    connect(cmd, &CreateCSRForCardKeyCommand::finished,
+            this, [this]() {
+                this->setEnabled(true);
+            });
+    cmd->start();
 }
