@@ -3,17 +3,15 @@
     This file is part of Kleopatra, the KDE keymanager
     SPDX-FileCopyrightText: 2017 Bundesamt für Sicherheit in der Informationstechnik
     SPDX-FileContributor: Intevation GmbH
-    SPDX-FileCopyrightText: 2020 g10 Code GmbH
+    SPDX-FileCopyrightText: 2020, 2022 g10 Code GmbH
     SPDX-FileContributor: Ingo Klöcker <dev@ingo-kloecker.de>
 
     SPDX-License-Identifier: GPL-2.0-or-later
 */
-#ifndef VIEW_PGPCARDWIDGET_H
-#define VIEW_PGPCARDWIDGET_H
+#pragma once
 
 #include "commands/changepincommand.h"
 
-#include <QMap>
 #include <QWidget>
 
 #include <gpgme++/error.h>
@@ -26,6 +24,7 @@ class QPushButton;
 namespace Kleo
 {
 class GenCardKeyDialog;
+class OpenPGPKeyCardWidget;
 
 namespace SmartCard
 {
@@ -43,12 +42,6 @@ public:
     void doGenKey(GenCardKeyDialog *dlg);
     void genKeyDone(const GpgME::Error &err, const std::string &backup);
 
-    struct KeyWidgets {
-        std::string keyGrip;
-        QLabel *keyFingerprint = nullptr;
-        QPushButton *createCSRButton = nullptr;
-    };
-
 public Q_SLOTS:
     void genkeyRequested();
     void changeNameRequested();
@@ -57,10 +50,9 @@ public Q_SLOTS:
     void changeUrlResult(const GpgME::Error &err);
     void createKeyFromCardKeys();
     void createCSR(const std::string &keyref);
+    void generateKey(const std::string &keyref);
 
 private:
-    KeyWidgets createKeyWidgets(const SmartCard::KeyPairInfo &keyInfo);
-    void updateKeyWidgets(const std::string &keyRef, const SmartCard::OpenPGPCard *card);
     void doChangePin(const std::string &keyRef, Commands::ChangePinCommand::ChangePinMode mode = Commands::ChangePinCommand::NormalMode);
 
 private:
@@ -69,7 +61,7 @@ private:
            *mVersionLabel = nullptr,
            *mUrlLabel = nullptr;
     QPushButton *mKeyForCardKeysButton = nullptr;
-    QMap<std::string, KeyWidgets> mKeyWidgets;
+    OpenPGPKeyCardWidget *mKeysWidget = nullptr;
     QString mUrl;
     bool mCardIsEmpty = false;
     bool mIs21 = false;
@@ -77,4 +69,3 @@ private:
 };
 } // namespace Kleo
 
-#endif // VIEW_PGPCARDWIDGET_H
